/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for FatFs     (C)ChaN, 2007        */
/*-----------------------------------------------------------------------*/
/* This is a stub disk I/O module that acts as front end of the existing */
/* disk I/O modules and attach it to FatFs module with common interface. */
/*-----------------------------------------------------------------------*/

#include "diskio.h"

/*-----------------------------------------------------------------------*/
/* Inidialize a Drive                                                    */

DSTATUS disk_initialize (
	BYTE drv				/* Physical drive nmuber (0..) */
)
{
	//mmcInit();
	return RES_OK;
}



/*-----------------------------------------------------------------------*/
/* Return Disk Status                                                    */

DSTATUS disk_status (
	BYTE drv		/* Physical drive nmuber (0..) */
)
{
	return RES_OK;
}



/*-----------------------------------------------------------------------*/
/* Read Sector(s)                                                        */

DRESULT disk_read (
	BYTE drv,		/* Physical drive nmuber (0..) */
	BYTE *buff,		/* Data buffer to store read data */
	DWORD sector,	/* Sector address (LBA) */
	BYTE count		/* Number of sectors to read (1..255) */
)
{
	#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
	//printf_P(PSTR("FAT: read %u sector from %lu to %04X \r\n"),count,sector,buff);
	#endif
	BYTE* p_to = buff;
	for(BYTE l=0;l<count;l++,p_to+=512)
	{
		if(usb_read_block(sector++,p_to)!=0)
			return RES_ERROR;

		#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
		/*
		if(gb_debug)
			for(uint8_t i=0;i<16;i++)
			{
				for(uint8_t j=0;j<32;j++)
					printf_P(PSTR("%02X "),*p_to++);
				puts_P(PSTR("\r\n"));
			}
		*/
		#endif
		#ifdef CFG_DEBUG_USB
		usb_main();
		#endif

	}
	return RES_OK;
}



/*-----------------------------------------------------------------------*/
/* Write Sector(s)                                                       */

#if _READONLY == 0
DRESULT disk_write (
	BYTE drv,			/* Physical drive nmuber (0..) */
	const BYTE *buff,	/* Data to be written */
	DWORD sector,		/* Sector address (LBA) */
	BYTE count			/* Number of sectors to write (1..255) */
)
{
	BYTE* p_to = buff;
	for(BYTE l=0;l<count;l++,p_to+=512)
	{
		if(usb_write_block(sector++,p_to)!=0)
			return RES_ERROR;
	}
	return RES_OK;
}
#endif /* _READONLY */



/*-----------------------------------------------------------------------*/
/* Miscellaneous Functions                                               */

DRESULT disk_ioctl (
	BYTE drv,		/* Physical drive nmuber (0..) */
	BYTE ctrl,		/* Control code */
	void *buff		/* Buffer to send/receive control data */
)
{
	if (ctrl == CTRL_SYNC)
	  return RES_OK;
	else
	  return RES_PARERR;
}


DWORD get_fattime (void)
{
	uint8_t Day=1,  Month=1,  Year=1970;
	uint8_t Hour=0, Minute=0, Second=0;

	//DS1307_GetDate(&Day,  &Month,  &Year);
	//DS1307_GetTime(&Hour, &Minute, &Second);


	return ((DWORD)(20 + Year) << 25) |
	             ((DWORD)Month << 21) |
	               ((DWORD)Day << 16) |
	              ((DWORD)Hour << 11) |
	             ((DWORD)Minute << 5) |
	      (((DWORD)Second >> 1) << 0);
}
