/*
=============================================================================
Smartport Virtual Hard Drive, version 0.99.0 (beta)


If you would like to be involved in this project, or simply get my 
documentation for your own smartport project, feel free to contact me at
pcedric@hotmail.com

Copyright 2010 Cédric PELTIER, pcedric@hotmail.com

This file is part of the Smartport VHD project

Smartport VHD is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 3

Smartport VHD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Smartport VHD. If not, see <http://www.gnu.org/licenses/>.
=============================================================================
*/

/*
             LUFA Library
     Copyright (C) Dean Camera, 2010.
              
  dean [at] fourwalledcubicle [dot] com
      www.fourwalledcubicle.com
*/

/*
  Copyright 2010  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, distribute, and sell this 
  software and its documentation for any purpose is hereby granted
  without fee, provided that the above copyright notice appear in 
  all copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting 
  documentation, and that the name of the author not be used in 
  advertising or publicity pertaining to distribution of the 
  software without specific, written prior permission.

  The author disclaim all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/




#include "spvhd.h"
#include "protocol.h"
#include "usb_serial.h"




/** Circular buffer to hold data from the host before it is processed by MainTask using standard IO functions */
RingBuff_t Host_to_Device_Buffer;

/** Circular buffer to hold data before it is sent to the host. */
RingBuff_t Device_to_Host_Buffer;

/** LUFA CDC Class driver interface configuration and state information. This structure is
 *  passed to all CDC Class driver functions, so that multiple instances of the same class
 *  within a device can be differentiated from one another.
 */
USB_ClassInfo_CDC_Device_t VirtualSerial_CDC_Interface =
{
    .Config = 
        {
            .ControlInterfaceNumber         = 0,

            .DataINEndpointNumber           = CDC_TX_EPNUM,
            .DataINEndpointSize             = CDC_TXRX_EPSIZE,
            .DataINEndpointDoubleBank       = false,

            .DataOUTEndpointNumber          = CDC_RX_EPNUM,
            .DataOUTEndpointSize            = CDC_TXRX_EPSIZE,
            .DataOUTEndpointDoubleBank      = false,

            .NotificationEndpointNumber     = CDC_NOTIFICATION_EPNUM,
            .NotificationEndpointSize       = CDC_NOTIFICATION_EPSIZE,
            .NotificationEndpointDoubleBank = false,
        },
};

/// execute pending usb tasks
void usb_main()
{
    /* Read bytes from the USB OUT endpoint into the local buffer */
    for (uint8_t DataBytesRem = CDC_Device_BytesReceived(&VirtualSerial_CDC_Interface); DataBytesRem != 0; DataBytesRem--)
    {
        if (!(BUFF_STATICSIZE - Host_to_Device_Buffer.Elements))
            break;
        Buffer_StoreElement(&Host_to_Device_Buffer, CDC_Device_ReceiveByte(&VirtualSerial_CDC_Interface));
    }

    /* Read bytes from the data buffer into the USB IN endpoint */
    while (Device_to_Host_Buffer.Elements)
        CDC_Device_SendByte(&VirtualSerial_CDC_Interface, Buffer_GetElement(&Device_to_Host_Buffer));

    CDC_Device_USBTask(&VirtualSerial_CDC_Interface);
    USB_USBTask();
}

void usb_init()
{
    USB_Init();
    // Initialize stdout and stdin for printf and scanf
    fdevopen(usb_sendData, usb_getData);
    Buffer_Initialize(&Host_to_Device_Buffer);
    Buffer_Initialize(&Device_to_Host_Buffer);
}

/** Event handler for the library USB Connection event. */
void EVENT_USB_Device_Connect(void)
{
    //LEDs_SetAllLEDs(LEDMASK_USB_ENUMERATING);
}

/** Event handler for the library USB Disconnection event. */
void EVENT_USB_Device_Disconnect(void)
{
    //LEDs_SetAllLEDs(LEDMASK_USB_NOTREADY);
}

/** Event handler for the library USB Configuration Changed event. */
void EVENT_USB_Device_ConfigurationChanged(void)
{
    //LEDs_SetAllLEDs(LEDMASK_USB_READY);

    //if (!(CDC_Device_ConfigureEndpoints(&VirtualSerial_CDC_Interface)))
    //  LEDs_SetAllLEDs(LEDMASK_USB_ERROR);
}

/** Event handler for the library USB Unhandled Control Request event. */
void EVENT_USB_Device_UnhandledControlRequest(void)
{
    CDC_Device_ProcessControlRequest(&VirtualSerial_CDC_Interface);
}

/** Event handler for the CDC Class driver Line Encoding Changed event.
 *
 *  \param[in] CDCInterfaceInfo  Pointer to the CDC class interface configuration structure being referenced
 */
void EVENT_CDC_Device_LineEncodingChanged(USB_ClassInfo_CDC_Device_t* const CDCInterfaceInfo)
{
    /* Not of any use for SRAM_Test
        Use Virtual Serial Port settings to control your application.
        Think of this as another data channel.  Use it for control/status messaging.
        Leave the data channel (MainTask putchar/getchar) for data only.
    

    switch (CDCInterfaceInfo->State.LineEncoding.ParityType)
    {
        case CDC_PARITY_Odd:
            // do something here
            break;
        case CDC_PARITY_Even:
            // and/or maybe here
            break;
        case CDC_PARITY_None:
            // maybe something here
            break;
        case CDC_PARITY_Mark:
            // something here could work
            break;
        case CDC_PARITY_Space:
            // you guessed it, something could go here
            break;
    }


    switch (CDCInterfaceInfo->State.LineEncoding.CharFormat)
    {
        case CDC_LINEENCODING_OneStopBit:
            // do something here
            break;
        case CDC_LINEENCODING_OneAndAHalfStopBits:
            // and/or maybe do something here
            break;
        case CDC_LINEENCODING_TwoStopBits:
            // something here could work
            break;
    }


    switch (CDCInterfaceInfo->State.LineEncoding.DataBits)
    {
        case 6:
            // possibly something
            break;
        case 7:
            // possibly something
            break;
        case 8:
            // possibly something
            break;
    }


    switch (CDCInterfaceInfo->State.LineEncoding.BaudRateBPS)
    {
        case 9600:
            // possibly something
            break;
        case 14400:
            // possibly something
            break;
        case 19200:
            // possibly something
            break;
        case 38400:
            // possibly something
            break;
        case 57600:
            // possibly something
            break;
        case 115200:
            // possibly something
            break;
    }
    */
}



/** CDC class driver event for a control line state change on a CDC interface. This event fires each time the host requests a
 *  control line state change (containing the virtual serial control line states, such as DTR). The new control line states
 *  are available in the ControlLineStates.HostToDevice value inside the CDC interface structure passed as a parameter, set as
 *  a mask of CDC_CONTROL_LINE_OUT_* masks.  1 is for 'Set'(Low) and 0 is for 'Clear'(High) as these are active low signals.
 *  \param[in,out] CDCInterfaceInfo  Pointer to a structure containing a CDC Class configuration and state
*/
void EVENT_CDC_Device_ControLineStateChanged(USB_ClassInfo_CDC_Device_t* const CDCInterfaceInfo)
{
    /* Not of any use for SRAM_Test
        Use Virtual Serial Port settings to control your application.
        Think of this as another data channel.  Use it for control/status messaging.
        Leave the data channel (MainTask putchar/getchar) for data only.
    

    if ((CDCInterfaceInfo->State.ControlLineStates.HostToDevice) & CDC_CONTROL_LINE_OUT_RTS) {
        // Host has set the RTS line
    } else {
        // Host has cleared the RTS line
    }

    if ((CDCInterfaceInfo->State.ControlLineStates.HostToDevice) & CDC_CONTROL_LINE_OUT_DTR) {
        // Host has set the DTR line
    } else {
        // Host has cleared the DTR line
    }
    */
}


/* Use this function to make sure data exists.
    Need this function as RingBuff will crash if getData is called on
    an empty buffer
*/
uint8_t usb_haveData(void) 
{
    return (uint8_t)(Host_to_Device_Buffer.Elements);
}

/* In order to use printf functions, need a function that will send data over
    the USB Virtual Serial Port link
    return 0 on success, else failure and ensure binary compatibility
*/
static int usb_sendData(char c, FILE *stream) {
    // most terminals require \r\n
    // however, do not include this conversion as it will break binary transfers 
    //if (c == '\n') {
    //  sendData('\r', stream);
    //}
    Buffer_StoreElement(&Device_to_Host_Buffer, (uint8_t)c);
    return 0;
}

/* Also require a function to receive data from the USB Virtual Serial Port link */
int usb_getData(FILE *__stream) 
{
    //if (something) return _FDEV_ERR; // cannot implement as GetElement has no error condition
    if (usb_haveData() == 0) {
        return _FDEV_EOF;
    }
    return (int)Buffer_GetElement(&Host_to_Device_Buffer);
}
