/*
=============================================================================
Smartport Virtual Hard Drive, version 0.99.0 (beta)


If you would like to be involved in this project, or simply get my 
documentation for your own smartport project, feel free to contact me at
pcedric@hotmail.com

Copyright 2010 Cédric PELTIER, pcedric@hotmail.com

This file is part of the Smartport VHD project

Smartport VHD is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 3

Smartport VHD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Smartport VHD. If not, see <http://www.gnu.org/licenses/>.
=============================================================================
*/

/*
             LUFA Library
     Copyright (C) Dean Camera, 2010.
              
  dean [at] fourwalledcubicle [dot] com
      www.fourwalledcubicle.com
*/

/*
  Copyright 2010  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, distribute, and sell this 
  software and its documentation for any purpose is hereby granted
  without fee, provided that the above copyright notice appear in 
  all copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting 
  documentation, and that the name of the author not be used in 
  advertising or publicity pertaining to distribution of the 
  software without specific, written prior permission.

  The author disclaim all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/


#include "spvhd.h"
#include "protocol.h"
#include "usb_storage.h"
#include "usb_storage_desc.h"

/** Index of the highest available LUN (Logical Unit) in the attached Mass Storage Device */
uint8_t MassStore_MaxLUNIndex;

/** Last error code */
uint8_t g_usb_last_error;
uint8_t g_usb_last_error_step;

/** Structure for the disk's capacity in blocks and block size */
SCSI_Capacity_t g_usb_disk_capacity;

/** Task to set the configuration of the attached device after it has been enumerated
 */
void usb_main(void)
{
    USB_USBTask();
}

/** Initialize usb layer and wait for the usb device */
void usb_init(uint8_t step)
{
    if(step==0)
    {
        #ifdef CFG_DEBUG_SERIAL
        if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR("USB: init\r\n"));
		#endif
        #ifdef USB_HOST_ONLY
        USB_Init();
        #else
        USB_Init(USB_MODE_HOST);
        #endif
        g_usb_last_error = 0;
        g_usb_last_error_step = 0;
    }
    USB_USBTask();
    switch (USB_HostState)
    {
    	  case HOST_STATE_WaitForDeviceRemoval:
//LEDs_SetAllLEDs(LEDMASK_POWER);
    	  	  break;
    	  case HOST_STATE_WaitForDevice:
            #ifdef CFG_DEBUG_SERIAL
//            puts_P(PSTR("USB: WaitForDevice\r\n"));
            #endif
    	  	  break;
    	  case HOST_STATE_Unattached:
            #ifdef CFG_DEBUG_SERIAL
//            puts_P(PSTR("USB: Unattached\r\n"));
            #endif
    	  	  break;
    	  case HOST_STATE_Powered:
            #ifdef CFG_DEBUG_SERIAL
            if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: Powered\r\n"));
            #endif
    	  	  break;
    	  case HOST_STATE_Powered_WaitForDeviceSettle:
            #ifdef CFG_DEBUG_SERIAL
//            puts_P(PSTR("USB: WaitForDeviceSettle\r\n"));
            #endif
    	  	  break;
    	  case HOST_STATE_Powered_WaitForConnect:
            #ifdef CFG_DEBUG_SERIAL
            if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: WaitForConnect\r\n"));
            #endif
    	  	  break;
    	  case HOST_STATE_Powered_DoReset:
            #ifdef CFG_DEBUG_SERIAL
            if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: DoReset\r\n"));
            #endif
    	  	  break;
    	  case HOST_STATE_Powered_ConfigPipe:
            #ifdef CFG_DEBUG_SERIAL
            if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: ConfigPipe\r\n"));
            #endif
    	  	  break;
    	  case HOST_STATE_Default:
            #ifdef CFG_DEBUG_SERIAL
            if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: Default\r\n"));
            #endif
    	  	  break;
    	  case HOST_STATE_Default_PostReset:
            #ifdef CFG_DEBUG_SERIAL
            if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: PostReset\r\n"));
            #endif
    	  	  break;
    	  case HOST_STATE_Default_PostAddressSet: 
            #ifdef CFG_DEBUG_SERIAL
            if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: PostAddressSet\r\n"));
            #endif
    	  	  break;

        case HOST_STATE_Addressed:
            #ifdef CFG_DEBUG_SERIAL
            if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: Getting Config Data.\r\n"));
            #endif
        
            /* Get and process the configuration descriptor data */
            if ((g_usb_last_error = ProcessConfigurationDescriptor()) != SuccessfulConfigRead)
            {
                #ifdef CFG_DEBUG_SERIAL
                if (g_usb_last_error == ControlError)
                {
                 if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: Control Error (Get Configuration).\r\n"));
                else
                 if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: Invalid Device.\r\n"));
                }
                 if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR("USB: Error Code: %d\r\n"), g_usb_last_error);
                
                #endif
                
                g_usb_last_error_step = 1;
                /* Wait until USB device disconnected */
                USB_HostState = HOST_STATE_WaitForDeviceRemoval;
                return;
            }

            /* Set the device configuration to the first configuration (rarely do devices use multiple configurations) */
            if ((g_usb_last_error = USB_Host_SetDeviceConfiguration(1)) != HOST_SENDCONTROL_Successful)
            {
                #ifdef CFG_DEBUG_SERIAL
                if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR( "USB: Control Error (Set Configuration).\r\nUSB: Error Code: %d\r\n" ), g_usb_last_error);
                #endif

                g_usb_last_error_step = 2;
                /* Wait until USB device disconnected */
                USB_HostState = HOST_STATE_WaitForDeviceRemoval;
                return;
            }
                
            #ifdef CFG_DEBUG_SERIAL
            if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: Mass Storage Disk Enumerated.\r\n"));
            #endif


            /* Send the request, display error and wait for device detach if request fails */
            if ((g_usb_last_error = MassStore_GetMaxLUN(&MassStore_MaxLUNIndex)) != HOST_SENDCONTROL_Successful)
            {   
                g_usb_last_error_step = 3;
                usb_DiskReadError(PSTR("USB: Get Max LUN"), g_usb_last_error);
                return;
            }
            
            #ifdef CFG_DEBUG_SERIAL
            /* Print number of LUNs detected in the attached device */
            if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR("USB: Total LUNs: %d - Using first LUN in device.\r\n"), (MassStore_MaxLUNIndex + 1));
            #endif
            
            /* Reset the Mass Storage device interface, ready for use */
            if ((g_usb_last_error = MassStore_MassStorageReset()) != HOST_SENDCONTROL_Successful)
            {
                g_usb_last_error_step = 4;
                usb_DiskReadError(PSTR("USB: Mass Storage Reset"), g_usb_last_error);
                return;
            }
            
            /* Get sense data from the device - many devices will not accept any other commands until the sense data
            * is read - both on start-up and after a failed command */
            SCSI_Request_Sense_Response_t SenseData;
            if ((g_usb_last_error = MassStore_RequestSense(0, &SenseData)) != 0)
            {
                g_usb_last_error_step = 5;
                // USB hard disk don't like this command, we should not consider this as a real error (lufa 110528)
					#ifdef CFG_DEBUG_SERIAL
					if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR("USB: Request Sense error %d\r\n"), g_usb_last_error);
					#endif
                //usb_DiskReadError(PSTR("USB: Request Sense"), g_usb_last_error);
                //break;
            }
            
            /* Set the prevent removal flag for the device, allowing it to be accessed */
            if ((g_usb_last_error = MassStore_PreventAllowMediumRemoval(0, true)) != 0)
            {
                g_usb_last_error_step = 6;
                // USB hard disk don't like this command, we should not consider this as a real error (lufa 100219 110528)
//                usb_DiskReadError(PSTR("USB: Prevent/Allow Medium Removal"), g_usb_last_error);
//                break;
            	#ifdef CFG_DEBUG_SERIAL
					if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR("USB: Prevent/Allow Medium Removal error %d\r\n"), g_usb_last_error);
					#endif
            }

            /* Get inquiry data from the device */
            SCSI_Inquiry_Response_t InquiryData;
            if ((g_usb_last_error = MassStore_Inquiry(0, &InquiryData)) != 0)
            {
                g_usb_last_error_step = 7;
					//#ifdef CFG_DEBUG_SERIAL
					//printf_P(PSTR("USB: Inquiry error %d\r\n"), g_usb_last_error);
					//MassStore_Inquiry_PrintResponse(&InquiryData);
					//#endif
                usb_DiskReadError(PSTR("USB: Inquiry"), g_usb_last_error);
                return;
            }
            else
            {
					#ifdef CFG_DEBUG_SERIAL
					/* Print vendor and product names of attached device */
					if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR("USB: Vendor \"%.8s\", Product \"%.16s\"\r\n"), InquiryData.VendorID, InquiryData.ProductID);   
					/* Wait until disk ready */
					//puts_P(PSTR("USB: Waiting until ready."));
					#endif
				}
				
            for (uint8_t i=0;i<10;i++)
            {
                #ifdef CFG_DEBUG_SERIAL
                if(gb_debug & CFG_DEBUG_MASK_USB) Serial_TxByte('.');
                #endif

                // Abort if device removed 
                if (USB_HostState == HOST_STATE_Unattached)
                	 return;

                // Check to see if the attached device is ready for new commands 
                g_usb_last_error = MassStore_TestUnitReady(0);
                
                // If attached device is ready, abort the loop
                if (!(g_usb_last_error))
                	 break;

                // If an error other than a logical command failure (indicating device busy) returned, abort 
/*                if (g_usb_last_error != MASS_STORE_SCSI_COMMAND_FAILED)
                {
                    g_usb_last_error_step = 8;
							#ifdef CFG_DEBUG_SERIAL
							printf_P(PSTR("USB: Test ready error code %d\r\n"), g_usb_last_error);
							#endif
                    //usb_DiskReadError(PSTR("\r\nUSB: Test Unit Ready"), g_usb_last_error);
                    //break;
                    // some device require more time to be ready and don't like to be harassed
                }
                _delay_ms(200);
*/
						if (g_usb_last_error != MS_ERROR_LOGICAL_CMD_FAILED)
						{
							g_usb_last_error_step = 9;
							#ifdef CFG_DEBUG_SERIAL
							if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR("USB: Test ready error code %d\r\n"), g_usb_last_error);
							#endif
							if(g_usb_last_error != 192)
							{
								// Reset the Mass Storage device interface, ready for use
								if ((g_usb_last_error = MassStore_MassStorageReset()) != HOST_SENDCONTROL_Successful)
								{
									 g_usb_last_error_step = 5;
									 usb_DiskReadError(PSTR("Mass Storage Reset"), g_usb_last_error);
									 return;
								}
								_delay_ms(1000);
							}
							else
							{
								_delay_ms(1000);
							}
/*							else
							{
								usb_DiskReadError(PSTR("Test Unit Ready"), g_usb_last_error);
								usb_init(0);
								return;
							}
*/						}
						else
						{
							SenseData.SenseKey = 255;
							if ((g_usb_last_error = MassStore_RequestSense(0, &SenseData)) != 0)
							{
								#ifdef CFG_DEBUG_SERIAL
								if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR("USB: Request sense error %d\r\n"), g_usb_last_error);
								#endif
							}
							if(SenseData.SenseKey!=0)
							{
								#ifdef CFG_DEBUG_SERIAL
								if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: Request sense delay\r\n"));
								#endif
								_delay_ms(1000);
							}
						}
            }
            
            #ifdef CFG_DEBUG_SERIAL
            if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("\r\nUSB: Retrieving Capacity...\r\n"));
            #endif
            
            // Retrieve disk capacity
            if ((g_usb_last_error = MassStore_ReadCapacity(0, &g_usb_disk_capacity)) != 0)
            {
                usb_DiskReadError(PSTR("Read Capacity"), g_usb_last_error);
				//	#ifdef CFG_DEBUG_SERIAL
				//	printf_P(PSTR("USB: Read capacity error code %d\r\n"), g_usb_last_error);
				//	#endif
                return;
            }
				else
				{
					if(g_usb_disk_capacity.BlockSize!=SP_BLOCKSIZE)
					{
						 usb_DiskReadError(PSTR("USB: block size isn't 512 bytes"), g_usb_last_error);
						 return;
					}
					// Display the disk capacity in blocks * block size bytes
					#ifdef CFG_DEBUG_SERIAL
					if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR("USB: %lu blocks of %lu bytes\r\n"), g_usb_disk_capacity.Blocks, g_usb_disk_capacity.BlockSize);
					#endif
				}
            
            /* Create a new buffer capable of holding a single block from the device */

            /* Read in the first 512 byte block from the device */
            /*
            if ((g_usb_last_error = MassStore_ReadDeviceBlock(0, 0x00000000, 1, SP_BLOCKSIZE, g_sp_buffer)) != 0)
            {
                usb_DiskReadError(PSTR("Read Device Block"), g_usb_last_error);
                USB_HostState = HOST_STATE_WaitForDeviceRemoval;
                break;
            }
            */
            //puts_P(PSTR("\r\nContents of first block:\r\n"));

            // Print out the first block in both HEX and ASCII, 16 bytes per line
            /*
            for (uint16_t Chunk = 0; Chunk < (g_usb_disk_capacity.BlockSize >> 4); Chunk++)
            {
                // Pointer to the start of the current 16-byte chunk in the read block of data
                uint8_t* ChunkPtr = &g_sp_buffer[Chunk << 4];
                
                // Print out the 16 bytes of the chunk in HEX format
                for (uint8_t ByteOffset = 0; ByteOffset < (1 << 4); ByteOffset++)
                {
                    char CurrByte = *(ChunkPtr + ByteOffset);
                
                    printf_P(PSTR("%.2X "), CurrByte);
                }
                
                puts_P(PSTR("    "));

                // Print out the 16 bytes of the chunk in ASCII format
                for (uint8_t ByteOffset = 0; ByteOffset < (1 << 4); ByteOffset++)
                {
                    char CurrByte = *(ChunkPtr + ByteOffset);
                
                    putchar(isprint(CurrByte) ? CurrByte : '.');
                }
                
                puts_P(PSTR("\r\n"));
            }
            */

            USB_HostState = HOST_STATE_Configured;
            break;
/*            
        case HOST_STATE_Configured:         
            break;
*/
    }
}

/** Read a block from the usb device 
@param address Address of the block to be read
@param p_dst Pointer to the buffer in the memory to store the block
@return 0 if the block has been read, -1 if an error occured
*/
int8_t usb_read_block(uint32_t address,uint8_t* p_dst)
{
    LEDs_TurnOnLEDs(LEDMASK_USB);
    if ((g_usb_last_error = MassStore_ReadDeviceBlock(0, address, 1, g_usb_disk_capacity.BlockSize, p_dst)) != 0)
    {
        usb_DiskReadError(PSTR("Read Device Block"), g_usb_last_error);
        USB_HostState = HOST_STATE_WaitForDeviceRemoval;
        LEDs_TurnOffLEDs(LEDMASK_USB);
        return -1;
    }
    LEDs_TurnOffLEDs(LEDMASK_USB);
    return 0;
}

/** Write a block to the usb device 
@param address Address of the block to be read
@param p_dst Pointer to the buffer in the memory to store the block
@return 0 if the block has been read, -1 if an error occured
*/
int8_t usb_write_block(uint32_t address,uint8_t* p_src)
{
    LEDs_TurnOnLEDs(LEDMASK_USB);
    if ((g_usb_last_error = MassStore_WriteDeviceBlock(0, address, 1, g_usb_disk_capacity.BlockSize, p_src)) != 0)
    {
        usb_DiskReadError(PSTR("Write Device Block"), g_usb_last_error);
        USB_HostState = HOST_STATE_WaitForDeviceRemoval;
        LEDs_TurnOffLEDs(LEDMASK_USB);
        return -1;
    }
    LEDs_TurnOffLEDs(LEDMASK_USB);
    return 0;
}
                                              
/** Event handler for the USB_DeviceAttached event. This indicates that a device has been attached to the host, and
 *  starts the library USB task to begin the enumeration and USB management process.
 */
void EVENT_USB_Host_DeviceAttached(void)
{
	/*
	g_sp_state.drive_max = 1;
   g_sp_state.initialized = 0;
   LEDs_TurnOnLEDs(LEDMASK_POWER);
 	*/
	#ifdef CFG_DEBUG_SERIAL
	if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: Device Attached\r\n"));
	#endif
}

/** Event handler for the USB_DeviceUnattached event. This indicates that a device has been removed from the host, and
 *  stops the library USB task management process.
 */
void EVENT_USB_Host_DeviceUnattached(void)
{
	/*
	g_sp_state.drive_max = 0;
   g_sp_state.initialized = 1;
   LEDs_TurnOnLEDs(LEDMASK_ERROR);
   */
	#ifdef CFG_DEBUG_SERIAL
	if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: Device Unattached\r\n"));
	#endif
}

/** Event handler for the USB_DeviceEnumerationComplete event. This indicates that a device has been successfully
 *  enumerated by the host and is now ready to be used by the application.
 */
void EVENT_USB_Host_DeviceEnumerationComplete(void)
{
	#ifdef CFG_DEBUG_SERIAL
	if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: Device Enumeration Complete\r\n"));
	#endif
}

/** Event handler for the USB_HostError event. This indicates that a hardware error occurred while in host mode. */
void EVENT_USB_Host_HostError(const uint8_t ErrorCode)
{
    USB_ShutDown();
	#ifdef CFG_DEBUG_SERIAL
	if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: hardware error\r\n"));
	#endif
    die(LEDMASK_USB,LEDMASK_ERROR,0,1000);
}

/** Event handler for the USB_DeviceEnumerationFailed event. This indicates that a problem occurred while
 *  enumerating an attached USB device.
 */
void EVENT_USB_Host_DeviceEnumerationFailed(const uint8_t ErrorCode, const uint8_t SubErrorCode)
{
	g_usb_last_error = ErrorCode;
	#ifdef CFG_DEBUG_SERIAL
	if(gb_debug & CFG_DEBUG_MASK_USB) puts_P(PSTR("USB: Enumeration failed\r\n"));
	#endif
}

/** Indicates that a communication error has occurred with the attached Mass Storage Device,
 *  printing error codes to the serial port and waiting until the device is removed before
 *  continuing.
 *
 *  \param[in] CommandString  ASCII string located in PROGMEM space indicating what operation failed
 *  \param[in] ErrorCode      Error code of the function which failed to complete successfully
 */
void usb_DiskReadError(char* CommandString, uint8_t ErrorCode)
{
    #ifdef CFG_DEBUG_SERIAL
    if (ErrorCode == MASS_STORE_SCSI_COMMAND_FAILED)
    {
        /* Display the error code */
        if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR( "USB: SCSI command error (%S)\r\n"), CommandString);
    }
    else
    {
        /* Display the error code */
        if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR( "USB: Command error (%S)\r\n"), CommandString);
        if(gb_debug & CFG_DEBUG_MASK_USB) printf_P(PSTR("USB: Error Code: %d\r\n" ), ErrorCode);  
    }
    #endif
    Pipe_Freeze();
    USB_HostState = HOST_STATE_WaitForDeviceRemoval;
}
