/*
=============================================================================
Smartport Virtual Hard Drive, version 0.99.0 (beta)
memory cache management source file

If you would like to be involved in this project, or simply get my 
documentation for your own smartport project, feel free to contact me at
pcedric@hotmail.com

Copyright 2010 Cédric PELTIER, pcedric@hotmail.com

This file is part of the Smartport VHD project

Smartport VHD is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 3

Smartport VHD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Smartport VHD. If not, see <http://www.gnu.org/licenses/>.
=============================================================================
*/




#include "spvhd.h"
#include "cache.h"
#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>
#include "extsram.h"
#ifdef CFG_DEBUG_SERIAL
#if(LUFA_VERSION==100219)
#include <LUFA/Drivers/Peripheral/SerialStream.h>
#else
#include <LUFA/Drivers/Peripheral/Serial.h>
#endif
#endif
#ifdef CFG_DEBUG_USB
	// using the usb port to debug the smartport protocol
	#include "usb_serial.h"
	#ifndef USB_CAN_BE_DEVICE
		#error USB_CAN_BE_DEVICE is not declared, something goes wrong...
	#endif
	extern USB_ClassInfo_CDC_Device_t VirtualSerial_CDC_Interface;
#endif
#include "Lib/FATFs/ff.h"

cacheset azerty;

#if CFG_CACHE != 2
	/// cache meta data in internal ram
	cacheset g_cacheline[CSET_MAX];
	/// cache buffer in internal ram 
	uint8_t g_cache_buffer[CSET_MAX*CSET_OBJECT_SIZE];
#else
	/// cache meta data in external ram
	cacheset* g_cacheline = CSET_BASE + (CSET_MAX * CSET_OBJECT_SIZE);
	/// cache buffer in external ram (see .h)
#endif

/// Initialize the cache line
void cset_init()
{
	#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
    if(gb_debug & CFG_DEBUG_MASK_CACHE)
        printf_P(PSTR("CSET: %u cacheset of %u bytes\r\n"),CSET_MAX,CSET_OBJECT_SIZE);
    #endif
	CSET_BANK_SET;
	cacheset* p_cacheline = g_cacheline;
	for(uint8_t i=CSET_MAX;i!=0;i--,p_cacheline++)
	{
		uint8_t j = i-1;
		p_cacheline->pointer = (uint8_t*)(CSET_BASE + CSET_OBJECT_SIZE * j);
		p_cacheline->id = 0xffffffff;
		p_cacheline->use = 0;
		p_cacheline->drive = 255;
		p_cacheline->canary = 0xCA;
		#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
		if(gb_debug & CFG_DEBUG_MASK_CACHE)
			printf_P(PSTR("CSET: object %u, id %lu, at %04X, buffer at %04X\r\n"),j,p_cacheline->id,(uint8_t*)(p_cacheline),p_cacheline->pointer);
		#endif
	}
	CSET_BANK_UNSET;
}

/// Look for the asked object
/// @param drive Drive number to look for
/// @param id Block number to look for
/// @param usage 0 to don't increment the counter usage, 1 normaly increment the counter usage
/// return a pointer on it or 0 if not found
uint8_t* cset_get(uint8_t drive,uint32_t id,uint8_t usage)
{
	#if CFG_CACHE != 0
	CSET_BANK_SET;
	cacheset* p_cacheline = g_cacheline;
	uint8_t* p_pointer = NULL;
	for(uint8_t i=CSET_MAX;i!=0;i--,p_cacheline++)
	{
		//uint8_t j = i-1;
		if(p_cacheline->canary != 0xCA)
		{
			#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
			if(gb_debug & CFG_DEBUG_MASK_CACHE)
				printf_P(PSTR("CSET: dead canary for cacheset %u !!!\r\n"),(i-1));
			#endif
        die(LEDMASK_SP,LEDMASK_SP|LEDMASK_ERROR,LEDMASK_SP|LEDMASK_ERROR,2000);	
		}
		if(p_cacheline->drive == drive && p_cacheline->id == id)
		{
			#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
			if(gb_debug & CFG_DEBUG_MASK_CACHE)
				printf_P(PSTR("CSET: found id %lu in object %u, use %u, at %04X\r\n"),id,(i-1),p_cacheline->use,p_cacheline->pointer);
			#endif
			if(p_cacheline->use <= 100)
				p_cacheline->use+=usage;
			p_pointer = p_cacheline->pointer; 
			CSET_BANK_UNSET;
			return p_pointer;
		}
	}
	CSET_BANK_UNSET;
	#endif
	return 0;
}

/// Find the least used object and replace it with the new one
/// @param drive Drive number to look for
/// @param id Block number to look for
/// @param usage 0 to don't increment the counter usage, 1 normaly increment the counter usage
/// return a pointer on the allocated buffer
uint8_t* cset_new(uint8_t drive, uint32_t id,uint8_t usage)
{
	//uint8_t elected = 0;
	cacheset* p_cacheline_elected=NULL;
	uint8_t* p_pointer = NULL;
	
	#if CFG_CACHE != 0	
	CSET_BANK_SET;
	uint8_t use_min = 255;
	cacheset* p_cacheline = g_cacheline;
	// parse the cache line to find an empty cache
	for(uint8_t i=CSET_MAX;i!=0;i--,p_cacheline++)
	{
		//uint8_t j = i-1;
		if(p_cacheline->canary != 0xCA)
		{
			#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
			if(gb_debug & CFG_DEBUG_MASK_CACHE)
				printf_P(PSTR("CSET: dead canary for cacheset %u !!!\r\n"),(i-1));
			#endif
        die(LEDMASK_SP,LEDMASK_SP|LEDMASK_ERROR,LEDMASK_SP|LEDMASK_ERROR,2000);	
		}
		if(p_cacheline->use == 0)
		{
			#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
			if(gb_debug & CFG_DEBUG_MASK_CACHE)
				printf_P(PSTR("CSET: create id %lu in object %u, use %u, at %04X\r\n"),id,(i-1),p_cacheline->use,p_cacheline->pointer);
			#endif
			p_cacheline->drive = drive;
			p_cacheline->id = id;
			p_cacheline->use = usage;
			p_pointer = p_cacheline->pointer; 
			CSET_BANK_UNSET;
			return p_pointer;
		}
		// locate the less used object in case there is no empty cache
		if(p_cacheline->use < use_min)
		{
			use_min = p_cacheline->use;
			p_cacheline_elected = p_cacheline;
		}
	}
	#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
	if(gb_debug & CFG_DEBUG_MASK_CACHE)
		printf_P(PSTR("CSET: create id %lu, use %u, at %04X\r\n"),id,p_cacheline_elected->use,p_cacheline_elected->pointer);
	#endif
	// Aging all old objects in the cache when no free cacheset are availables
	// to allow new one to take place
	if(use_min>1)
	{
		--use_min;
		#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
		if(gb_debug & CFG_DEBUG_MASK_CACHE)
			printf_P(PSTR("CSET: aging the cache line by %d\r\n"),use_min);
		#endif
		p_cacheline = g_cacheline;
		for(uint8_t i=CSET_MAX;i!=0;i--,p_cacheline++)
			p_cacheline->use -= use_min;
	}
	#else
	
	p_cacheline_elected = g_cacheline;
	
	#endif
	
	p_cacheline_elected->drive = drive;
	p_cacheline_elected->id = id;
	p_cacheline_elected->use = usage;
	p_pointer = p_cacheline_elected->pointer; 
	CSET_BANK_UNSET;
	return p_pointer;
}
