/*
=============================================================================
Smartport Virtual Hard Drive, version 0.99.0 (beta)
memory cache management source file

If you would like to be involved in this project, or simply get my 
documentation for your own smartport project, feel free to contact me at
pcedric@hotmail.com

Copyright 2010 Cédric PELTIER, pcedric@hotmail.com

This file is part of the Smartport VHD project

Smartport VHD is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 3

Smartport VHD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Smartport VHD. If not, see <http://www.gnu.org/licenses/>.
=============================================================================
*/

#ifndef _CACHESET_H_
#define _CACHESET_H_
    #include <stdint.h>
    #include "extsram.h"
    
	/// size of objects to cache
	#define CSET_OBJECT_SIZE 512

	#ifndef CFG_CACHE
		#error CFG_CACHE not defined, must 0, 1 or 2
	#endif
	
	#if CFG_CACHE == 0
		/// base address of the memory cache
		#define CSET_BASE g_cache_buffer
		/// maximum number of object to cache limited to only one
		#define CSET_MAX 1
		/// memory bank swapping unused
		#define CSET_BANK_SET
		#define CSET_BANK_UNSET
		/// cache buffer in internal ram 
		uint8_t g_cache_buffer[CSET_MAX*CSET_OBJECT_SIZE];
	#endif
	
	#if CFG_CACHE == 1
		/// base address of the memory cache
		#define CSET_BASE g_cache_buffer
		#if MCU == at90usb647
		/// maximum number of object to cache for a 647 = 3
		#define CSET_MAX 3
		#endif
		#if MCU == at90usb1247
		/// maximum number of object to cache for a 1287 = 11
		#define CSET_MAX 8
		#endif
		/// memory bank swapping unused
		#define CSET_BANK_SET
		#define CSET_BANK_UNSET
		/// cache buffer in internal ram 
		uint8_t g_cache_buffer[CSET_MAX*CSET_OBJECT_SIZE];
	#endif

	#if CFG_CACHE == 2
		/// cache buffer in external static ram 
		/// base address of the memory cache
		#define CSET_BASE EXT_SRAM_START
		/// maximum number of object to cache
		#define CSET_MAX 8
		//(EXT_SRAM_SIZE / ( CSET_OBJECT_SIZE + sizeof(cacheset) ) )
		/// macro to select the righ bank of memory when external memory is available
		#define CSET_BANK_SET 	SELECT_EXTSRAM_BANK1
		#define CSET_BANK_UNSET	SELECT_EXTSRAM_BANK0
	#endif

	/// cache meta data unit structure 
	typedef struct struct_cacheset 
	{
		/// drive number
		uint8_t drive;
		/// block address
		uint32_t id;
		/// number of calls to this block
		uint8_t use;
		/// pointer in memory to the storage of the block content
		uint8_t* pointer;
		/// sanity check
		uint8_t canary;
	} cacheset;

	void cset_init(void);
	uint8_t* cset_get(uint8_t drive, uint32_t id, uint8_t usage);
	uint8_t* cset_new(uint8_t drive, uint32_t id, uint8_t usage);
	

#endif
