/*
=============================================================================
Smartport Virtual Hard Drive, version 1.00.0

Copyright 2010-2012 Cédric PELTIER, pcedric@hotmail.com

This file is part of the Smartport VHD project

Smartport VHD is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 3

Smartport VHD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Smartport VHD. If not, see <http://www.gnu.org/licenses/>.
=============================================================================
*/

#include "ds1307.h"
/*
          49041 ($BF91)     49040 ($BF90)

         7 6 5 4 3 2 1 0   7 6 5 4 3 2 1 0
        +-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+-+-+
 DATE:  |    year     |  month  |   day   |
        +-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+-+-+

         7 6 5 4 3 2 1 0   7 6 5 4 3 2 1 0
        +-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+-+-+
 TIME:  |    hour       | |    minute     |
        +-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+-+-+

          49043 ($BF93)     49042 ($BF92)
*/

/// Read the clock from the RTC
bool ds1307_read(uint8_t* lp_ptr)
{
   uint8_t l_data;
   uint8_t l_rc;
   
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
        printf_P(PSTR("RTC: ds1307_read(%04X)\r\n"),lp_ptr);
    #endif

   l_rc = i2c_init(DS1307_ID,DS1307_ADDR,TW_WRITE);
   if(l_rc)
   	l_rc = i2c_write(0x00);
   i2c_stop();

   l_rc = i2c_init(DS1307_ID,DS1307_ADDR,TW_READ);

   // Read the Second Register, Send Master Acknowledge
   if(l_rc)
   	l_rc = i2c_read(&lp_ptr[0],true);
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
        printf_P(PSTR("RTC: seconds %02X\r\n"),lp_ptr[0]);
    #endif
 
   if(l_rc)
   	l_rc = i2c_read(&lp_ptr[1],true);
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
        printf_P(PSTR("RTC: minutess %02X\r\n"),lp_ptr[1]);
    #endif

   if(l_rc)
   	l_rc = i2c_read(&l_data,true);
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
        printf_P(PSTR("RTC: hours %02X\r\n"),l_data);
    #endif
   if ((l_data & 0x40) == 0x40)
   {
		lp_ptr[7] = DS1307_12H;
		lp_ptr[8] = (l_data & 0x20) >> 5;   // ampm_mode: 0-AM, 1-PM
		lp_ptr[2] = l_data & 0x1F;
   }
   else
   {
		lp_ptr[7] = DS1307_24H;
		lp_ptr[8] = 0;
		lp_ptr[2] = l_data & 0x3F;
   }

   if(l_rc)
   	l_rc = i2c_read(&lp_ptr[3],true);
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
        printf_P(PSTR("RTC: day of week %02X\r\n"),lp_ptr[3]);
    #endif

   if(l_rc)
   	l_rc = i2c_read(&lp_ptr[4],true);
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
        printf_P(PSTR("RTC: day   %02X\r\n"),lp_ptr[4]);
    #endif

   if(l_rc)
   	l_rc = i2c_read(&lp_ptr[5],true);
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
        printf_P(PSTR("RTC: month %02X\r\n"),lp_ptr[5]);
    #endif

   // Read the Year Register, Send Master No Acknowledge
   if(l_rc)
   	l_rc = i2c_read(&lp_ptr[6],false);
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
        printf_P(PSTR("RTC: year  %02X\r\n"),lp_ptr[6]);
    #endif

   i2c_stop();
   return l_rc;
}

/// Write the clock to the RTC
bool ds1307_write(uint8_t* lp_ptr)
{
	uint8_t l_data;
   uint8_t l_rc;
	
	#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
   	printf_P(PSTR("RTC: ds1307_write(%04X)\r\n"),lp_ptr);
   #endif

   l_rc = i2c_init(DS1307_ID,DS1307_ADDR,TW_WRITE);
   if(l_rc)
   	l_rc = i2c_write(0x00);
   
   // enabled oscillator and advertisely reset seconds
   if(l_rc)
   	l_rc = i2c_write(0);
   
   // set minutes
   l_data = lp_ptr[1];
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
   	printf_P(PSTR("RTC: minutes %02X\r\n"),l_data);
   #endif
   if(l_rc)
   	l_rc = i2c_write(l_data);
   
   // set hours in 24 hours mode
   l_data = lp_ptr[2] & 0x3F;
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
   	printf_P(PSTR("RTC: hours %02X\r\n"),l_data);
   #endif
   if(l_rc)
   	l_rc = i2c_write(l_data);
   i2c_stop();
   
   if(l_rc)
   	l_rc = i2c_init(DS1307_ID,DS1307_ADDR,TW_WRITE);
   if(l_rc)
   	l_rc = i2c_write(0x04);
   
   l_data = lp_ptr[4];
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
   	printf_P(PSTR("RTC: day   %02X\r\n"),l_data);
   #endif
   if(l_rc)
   	l_rc = i2c_write(l_data);
 
   l_data = lp_ptr[5];
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
   	printf_P(PSTR("RTC: month %02X\r\n"),l_data);
   #endif
   if(l_rc)
   	l_rc = i2c_write(l_data);
   
   l_data = lp_ptr[6];
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
   	printf_P(PSTR("RTC: year  %02X\r\n"),l_data);
   #endif
   if(l_rc)
   	l_rc = i2c_write(l_data);
   i2c_stop();
   return l_rc;
}

/// Binary to BCD
uint8_t bin2bcd(uint8_t l_bin)
{
  uint8_t l_upper;
  uint8_t l_lower;
  uint8_t l_data;
  l_upper = l_bin / 10;
  l_lower = l_bin - l_upper * 10;
  l_data = (l_upper << 4) | l_lower;
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
   	printf_P(PSTR("RTC: %u => %02X\r\n"),l_bin,l_data);
   #endif
  return l_data;
}

/// BCD to binary
uint8_t bcd2bin(uint8_t l_bcd)
{
  uint8_t l_data;
  l_data = (l_bcd >> 4) * 10 + (l_bcd & 15);
   #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
   if(gb_debug & CFG_DEBUG_MASK_RTC)
   	printf_P(PSTR("RTC: %02X => %u\r\n"),l_bcd,l_data);
   #endif
  return l_data;
}

