/*
=============================================================================
Smartport Virtual Hard Drive, version 0.99.0 (beta)
external ram routines

If you would like to be involved in this project, or simply get my 
documentation for your own smartport project, feel free to contact me at
pcedric@hotmail.com

Copyright 2010 Cédric PELTIER, pcedric@hotmail.com

This file is part of the Smartport VHD project

Smartport VHD is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 3

Smartport VHD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Smartport VHD. If not, see <http://www.gnu.org/licenses/>.
=============================================================================
*/




#include "spvhd.h"
#include "protocol.h"
#include "extsram.h"
#ifdef CFG_DEBUG_USB
#include "usb_serial.h"
#endif



/// Global pointer to the start of external memory
static uint8_t* ExtMemArray = (uint8_t*)EXT_SRAM_START;
static uint16_t extSramStart = (uint16_t)EXT_SRAM_START;
static uint16_t extSramEnd = (uint16_t)EXT_SRAM_END;

volatile uint16_t EMAindex = 0;
volatile uint16_t firstSRAMfailAddress = 0;
volatile uint16_t SRAMfailAddress[4] = {0,0,0,0};
volatile uint8_t SRAMTestStatus[4] = {0,0,0,0};
volatile uint8_t FillingRAM = 0;
volatile uint8_t currentHelpPage = 0;

/// Initialize board to acess external sram and check the sram
/// If something is wrong with the sram, simply die
void xr_init()
{
#if CFG_XRAM == 1
    #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
    printf_P(PSTR("RAM: start %04X end %04X banks %d\r\n"),EXT_SRAM_START,EXT_SRAM_END,SRAM_BANKS);
    #endif
    
    // Enable and check External SRAM
    // set PortE for output
    // PE6 is chip enable (^CS - active low)
    // PE7 is highest address bit (bank)
    // PE0,1,2 are SRAM-related
    // PE3,4,5 are availables
    DDRE |= ((0 << PE6) | (0 << PE7) | (1 << PE0) | (1 << PE1) | (1 << PE2));  
    PORTE |= ((0 << PE6) | (0 << PE7) | (1 << PE0) | (1 << PE1) | (1 << PE2));
    XMCRA = ((1 << SRE));  // enable external SRAM with 0 wait states
    #if (SRAM_BANKS==2)
      XMCRB = 0; // using the whole port C to address automatically external sram
    #endif
    #if (SRAM_BANKS==4)
      XMCRB = 1; // PC7 is manually controlled to switch between four banks of 32KB
    #endif
     #ifdef SRAM_TEST
    for(int8_t i=(int8_t)SRAM_BANKS-1;i>=0;i--)
    {
      switch(i)
      {
        case 0:
          SELECT_EXTSRAM_BANK0;
          break;
        case 1:
          SELECT_EXTSRAM_BANK1;
          break;
        case 2:
          SELECT_EXTSRAM_BANK2;
          break;
        case 3:
          SELECT_EXTSRAM_BANK3;
          break;        
      }
      SRAMTestStatus[i] = xr_init_test(ExtMemArray, extSramStart, extSramEnd);
      if(SRAMTestStatus[i]!=0)
      {
        #if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
        printf_P(PSTR("RAM: fail address %04X\r\n"),firstSRAMfailAddress);
        #endif
          die(LEDMASK_HDW,LEDMASK_HDW|LEDMASK_ERROR,LEDMASK_HDW|LEDMASK_ERROR,1000);
          //LEDs_SetAllLEDs(LEDMASK_ERROR);
          //SRAMfailAddress[i]=firstSRAMfailAddress;
      }
    }
    #endif
    SELECT_EXTSRAM_BANK0;
#endif
}

/// Test the external sram zone, thank to the sram test demo
uint8_t xr_init_test(uint8_t* array, uint16_t startAddress, uint16_t endAddress)
{
#if CFG_XRAM == 1
    uint16_t i = 0;
    uint16_t ArraySize = (uint16_t)(endAddress - startAddress);

    // Test1 - test upward count
    for (i = 0; i < ArraySize; i++) {
            array[i] = (uint8_t)i;
    }
    for (i = 0; i < ArraySize; i++) {
            if (array[i] != (uint8_t)i) {
                    firstSRAMfailAddress = (uint16_t)(&array[i]);
                    return 1;
            }
    }

    // Test2 - test downward count
    for (i = ArraySize; i > 0; i--) {
            array[i] = (uint8_t)i;
    }
    for (i = ArraySize; i > 0; i--) {
            if (array[i] != (uint8_t)i) {
                    firstSRAMfailAddress = (uint16_t)(&array[i]);
                    return 2;
            }
    }

    // Test3 - test 0xFF = 0b11111111
    for (i = 0; i < ArraySize; i++) {
            array[i] = 0xFF;
    }
    for (i = 0; i < ArraySize; i++) {
            if (array[i] != 0xFF) {
                    firstSRAMfailAddress = (uint16_t)(&array[i]);
                    return 3;
            }
    }       

    // Test4 - test 0xFF = 0b00000000
    for (i = 0; i < ArraySize; i++) {
            array[i] = 0x00;
    }
    for (i = 0; i < ArraySize; i++) {
            if (array[i] != 0x00) {
                    firstSRAMfailAddress = (uint16_t)(&array[i]);
                    return 4;
            }
    }       

    // Test5 - test 0xAA = 0b10101010
    for (i = 0; i < ArraySize; i++) {
            array[i] = 0xAA;
    }
    for (i = 0; i < ArraySize; i++) {
            if (array[i] != 0xAA) {
                    firstSRAMfailAddress = (uint16_t)(&array[i]);
                    return 5;
            }
    }       

    // Test 6 - test 0x55 = 0b01010101
    for (i = 0; i < ArraySize; i++) {
            array[i] = 0x55;
    }
    for (i = 0; i < ArraySize; i++) {
            if (array[i] != 0x55) {
                    firstSRAMfailAddress = (uint16_t)(&array[i]);
                    return 6;
            }
    }       

    // Test7 - test upward count again
    for (i = 0; i < ArraySize; i++) {
            array[i] = (uint8_t)i;
    }
    for (i = 0; i < ArraySize; i++) {
            if (array[i] != (uint8_t)i) {
                    firstSRAMfailAddress = (uint16_t)(&array[i]);
                    return 7;
            }
    }

#endif
    return 0;
}

/// External sram interaction trough serial usb, thank to the sram test demo
void xr_main(void)
{
#if CFG_XRAM == 1
#ifdef CFG_DEBUG_USB
    uint8_t  tempByte1 = 0;
    uint8_t  tempByte2 = 0;
    uint8_t  tempByte3 = 0;
    uint16_t  tempWord = 0;
    uint8_t*  tempBytePtr = 0;

    while (usb_haveData()) {
        tempByte1 = getchar();

        if (tempByte1 == 'n') { // get the next byte of data from external SRAM array - use i cmd to get first
            printf("%3u\r\n", ExtMemArray[EMAindex]);
            EMAindex++;
        } else if ((tempByte1 == 'h') || (tempByte1 == '?')) { // print help
            if (currentHelpPage == 0) {
                printf_P(PSTR("\nAvailable Commands\r\n"));
                printf_P(PSTR("o - print info\r\n"));
                printf_P(PSTR("\tPress h again for next page\r\n"));
                currentHelpPage = 1;
            } else if (currentHelpPage == 1) {
                printf_P(PSTR("t - SRAM test status\r\n"));
                printf_P(PSTR("s - size of SRAM\r\n"));
                printf_P(PSTR("\tPress h again for next page\r\n"));
                currentHelpPage = 2;
            } else if (currentHelpPage == 2) {
                printf_P(PSTR("b - current bank\r\n"));
                printf_P(PSTR("0 - select bank 0\r\n"));
                printf_P(PSTR("\tPress h again for next page\r\n"));
                currentHelpPage = 3;
            } else if (currentHelpPage == 3) {
                printf_P(PSTR("1 - select bank 1\r\n"));
                printf_P(PSTR("f??? - fill array at index ?? with data byte ? and return status\r\n"));
                printf_P(PSTR("\tPress h again for next page\r\n"));
                currentHelpPage = 4;
            } else if (currentHelpPage == 4) {
                printf_P(PSTR("g??? - fill array at index ?? with data byte ? but no status msg\r\n"));
                printf_P(PSTR("a?? - data at address ?? \r\n"));
                printf_P(PSTR("\tPress h again for next page\r\n"));
                currentHelpPage = 5;
            } else {
                printf_P(PSTR("i?? - data at SRAM array index ?? \r\n"));
                printf_P(PSTR("n - data at next SRAM array index - set start index with i?? \r\n"));
                printf_P(PSTR("\tThis is the end of the help page.\r\n"));
                currentHelpPage = 0;
            }
        } else if (tempByte1 == 'o') { // print info regarding SRAM
            printf_P(PSTR("SRAM StartAddr = "));
            printf("%5u,", (uint16_t)EXT_SRAM_START);
            printf_P(PSTR(" EndAddr = "));
            printf("%5u,", (uint16_t)EXT_SRAM_END);
            printf_P(PSTR(" Size = "));
            printf("%5u", (uint16_t)EXT_SRAM_SIZE);
            printf_P(PSTR(" bytes\r\n"));
        } else if (tempByte1 == 't') { // print whether SRAM failed internal testing
            for(int8_t i=(int8_t)SRAM_BANKS-1;i>=0;i--)
            {
                if (SRAMTestStatus[i]!=0)
                {
                    printf("Bank %u, error code %u at address %5u\r\n", i, SRAMTestStatus[i],SRAMfailAddress[i]);
                }   
                else
                {
                    printf("Bank %u, ok\r\n",i);
                }
            }
        } else if (tempByte1 == 's') { // size of SRAM
            printf("%5u\r\n", ((uint16_t)EXT_SRAM_SIZE));
        } else if (tempByte1 == 'f') { // start filling SRAM, each subsequent byte will fill next index, need to fill all of it
            tempByte2 = getchar();
            tempByte3 = getchar();
            tempByte1 = getchar();
            tempWord = (((uint16_t)tempByte2 << 8) + ((uint16_t)tempByte3));
            ExtMemArray[tempWord] = tempByte1;
            printf_P(PSTR("Filled EMA["));
            printf("%5u", tempWord);
            printf_P(PSTR("] with "));
            printf("%3u\r\n", ExtMemArray[tempWord]);
        } else if (tempByte1 == 'g') { // start filling SRAM, each subsequent byte will fill next index, need to fill all of it
            tempByte2 = getchar();
            tempByte3 = getchar();
            tempByte1 = getchar();
            tempWord = (((uint16_t)tempByte2 << 8) + ((uint16_t)tempByte3));
            ExtMemArray[tempWord] = tempByte1;
        } else if (tempByte1 == 'b') { // print current SRAM bank
            printf_P(PSTR("SRAM_Bank = "));
            printf("%u\r\n", (PINE >> 7));
        } else if (tempByte1 == '0') { // select SRAM bank 0 for usage
            SELECT_EXTSRAM_BANK0;
            printf("Bank 0 selected\r\n");
        } else if (tempByte1 == '1') { // select SRAM bank 1 for usage
            SELECT_EXTSRAM_BANK1;
            printf("Bank 1 selected\r\n");
        } else if (tempByte1 == '2') { // select SRAM bank 2 for usage
            SELECT_EXTSRAM_BANK2;
            printf("Bank 2 selected\r\n");
        } else if (tempByte1 == '3') { // select SRAM bank 3 for usage
            SELECT_EXTSRAM_BANK3;
            printf("Bank 3 selected\r\n");
        } else if (tempByte1 == 'a') { // get the data byte from memory location defined by subsequent 2 bytes (word)
            tempByte2 = getchar();
            tempByte3 = getchar();
            tempWord = (((uint16_t)tempByte2 << 8) + ((uint16_t)tempByte3));
            tempBytePtr = (uint8_t*)tempWord;
            printf("Addr: %5u: = %3u\r\n", tempWord, ((uint8_t)(*tempBytePtr)));
        } else if (tempByte1 == 'i') { // get the data byte from ExtMemArray at index defined by subsequent 2 bytes (word)
            tempByte2 = getchar();
            tempByte3 = getchar();
            tempWord = (((uint16_t)tempByte2 << 8) + ((uint16_t)tempByte3));
            EMAindex = tempWord;
            printf_P(PSTR("EMA["));
            printf("%5u", tempWord);
            printf_P(PSTR("] = Addr:"));
            printf("%5u = %3u\r\n", ((uint16_t)&ExtMemArray[EMAindex]), ExtMemArray[EMAindex]);
        } else {
            //printf("%3u", (uint8_t)'f');
            //printf_P(PSTR("-ERROR\r\n"));
            //printf(" ");
        }
    }
#endif
#endif
}

