/**
=============================================================================
Smartport Virtual Hard Drive, version 0.99.0 (beta)
smartport (aka Cbus) protocol interpreter debug routines

If you would like to be involved in this project, or simply get my 
documentation for your own smartport project, feel free to contact me at
pcedric@hotmail.com

Copyright 2010 Cédric PELTIER, pcedric@hotmail.com

This file is part of the Smartport VHD project

Smartport VHD is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 3

Smartport VHD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Smartport VHD. If not, see <http://www.gnu.org/licenses/>.
=============================================================================
*/

#include "spvhd.h"
#include "protocol.h"
#include "cache.h"
#include "ds1307.h"
#include <stdio.h>
#include <string.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#ifdef CFG_DEBUG_USB
#include "usb_serial.h"
#endif
#ifdef CFG_DEBUG_SERIAL
#include <LUFA/Drivers/Peripheral/Serial.h>
#endif

#ifdef CFG_DEBUG_PUSH
/** 
Log any line change
*/
void sp_debug_lines()
{
	static uint8_t s_oack;
	static uint8_t s_idr2;
	static uint8_t s_odr2;
	static uint8_t s_ilstrb;
	static uint8_t s_ihdsel;
	static uint8_t s_ica2;
	static uint8_t s_ica1;
	static uint8_t s_ireq;
    uint8_t l_line;
    uint8_t l_change = 0;
    
    l_line = SP_IF_OACK;
    if(l_line != s_oack)
    {
    	if(l_change==0)
    	{
    		SP_DEBUG_PUSH('(');
    		l_change=1;
    	}
        SP_DEBUG_PUSH((l_line?'A':'a'));
        s_oack=l_line;
    }
    
    l_line = SP_IF_IDR2;
    if(l_line != s_idr2)
    {
    	if(l_change==0)
    	{
    		SP_DEBUG_PUSH('(');
    		l_change=1;
    	}
        SP_DEBUG_PUSH((l_line?'V':'v'));
        s_idr2=l_line;
    }
    
    l_line = SP_IF_ODR2;
    if(l_line != s_odr2)
    {
    	if(l_change==0)
    	{
    		SP_DEBUG_PUSH('(');
    		l_change=1;
    	}
        SP_DEBUG_PUSH((l_line?'W':'w'));
        s_odr2=l_line;
    }
    
    l_line = SP_IF_ILSTRB;
    if(l_line != s_ilstrb)
    {
    	if(l_change==0)
    	{
    		SP_DEBUG_PUSH('(');
    		l_change=1;
    	}
        SP_DEBUG_PUSH((l_line?'R':'r'));
        s_ilstrb=l_line;
    }
    
    l_line = SP_IF_IHDSEL;
    if(l_line != s_ihdsel)
    {
    	if(l_change==0)
    	{
    		SP_DEBUG_PUSH('(');
    		l_change=1;
    	}
        SP_DEBUG_PUSH((l_line?'S':'s'));
        s_ihdsel=l_line;
    }
    
    l_line = SP_IF_ICA2;
    if(l_line != s_ica2)
    {
    	if(l_change==0)
    	{
    		SP_DEBUG_PUSH('(');
    		l_change=1;
    	}
        SP_DEBUG_PUSH((l_line?'Q':'q'));
        s_ica2=l_line;
    }
    
    l_line = SP_IF_ICA1;
    if(l_line != s_ica1)
    {
    	if(l_change==0)
    	{
    		SP_DEBUG_PUSH('(');
    		l_change=1;
    	}
        SP_DEBUG_PUSH((l_line?'P':'p'));
        s_ica1=l_line;
    }
    
    l_line = SP_IF_IREQ;
    if(l_line != s_ireq)
    {
    	if(l_change==0)
    	{
    		SP_DEBUG_PUSH('(');
    		l_change=1;
    	}
        SP_DEBUG_PUSH((l_line?'O':'o'));
        s_ireq=l_line;
    }
    
	if(l_change==1)
		SP_DEBUG_PUSH(')');
}

/**
Write a char into the debug buffer
*/
void sp_debug_push(uint8_t chr)
{
    if(g_sp_debug_pos<SP_DEBUG_BUF_SIZE)
        g_sp_debug[g_sp_debug_pos++]=chr;
}
#endif

/**
Manage communications for debugging purpose
*/
void sp_debug()
{
#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
    uint8_t c;
#ifdef CFG_DEBUG_USB
    while (usb_haveData())
    {
        c = getchar();
#endif
#ifdef CFG_DEBUG_SERIAL
    while (Serial_IsCharReceived())
    {
        c = getchar();
#endif
        uint16_t tcnt1;
        uint16_t icr1;
        uint16_t ocr1a;
        uint16_t ocr1b;
        uint16_t ocr1c;
        uint16_t tccr0a;
        uint16_t tccr1a;
        uint16_t tccr1b;
        uint16_t timsk1;
        //uint8_t delta;
        uint16_t p;
        uint8_t l_tmp[10];
        switch(c)
        {
            case '1':
                gb_debug  ^= CFG_DEBUG_MASK_CACHE;
                if(gb_debug & CFG_DEBUG_MASK_CACHE)
                    puts_P(PSTR("debug cache\r\n"));
                break;
            case '2':
                gb_debug  ^= CFG_DEBUG_MASK_RCV;
                if(gb_debug & CFG_DEBUG_MASK_RCV)
                    puts_P(PSTR("debug protocol receive\r\n"));
                break;
            case '3':
                gb_debug  ^= CFG_DEBUG_MASK_SND;
                if(gb_debug & CFG_DEBUG_MASK_SND)
                    puts_P(PSTR("debug protocol send\r\n"));
                break;
            case '4':
                gb_debug  ^= CFG_DEBUG_MASK_PROTO;
                if(gb_debug & CFG_DEBUG_MASK_PROTO)
                    puts_P(PSTR("debug protocol\r\n"));
                break;
            case '5':
                gb_debug  ^= CFG_DEBUG_MASK_SPVHD;
                if(gb_debug & CFG_DEBUG_MASK_SPVHD)
                    puts_P(PSTR("debug spvhd\r\n"));
                break;
            case '6':
                gb_debug  ^= CFG_DEBUG_MASK_BUS;
                if(gb_debug & CFG_DEBUG_MASK_BUS)
                    puts_P(PSTR("debug bus\r\n"));
                break;
            case '7':
                gb_debug  ^= CFG_DEBUG_MASK_USB;
                if(gb_debug & CFG_DEBUG_MASK_USB)
                    puts_P(PSTR("debug usb\r\n"));
                break;
            case '8':
                gb_debug  ^= CFG_DEBUG_MASK_RTC;
                if(gb_debug & CFG_DEBUG_MASK_RTC)
                    puts_P(PSTR("debug rtc\r\n"));
                break;
            case '9':
                gb_debug  ^= CFG_DEBUG_MASK_I2C;
                if(gb_debug & CFG_DEBUG_MASK_I2C)
                    puts_P(PSTR("debug i2c\r\n"));
                break;
            case '0':
                gb_debug  ^= CFG_DEBUG_MASK_RWTS;
                if(gb_debug & CFG_DEBUG_MASK_RWTS)
                    puts_P(PSTR("debug rwts\r\n"));
                break;
            case 'o':
                // ports dump
                printf_P(PSTR(" ddrb=%02X  ddrd=%02X  ddrf=%02X\r\n"),DDRB,DDRD,DDRF);
                printf_P(PSTR("portb=%02X portd=%02X portf=%02X\r\n"),PINB,PIND,PINF);
                #ifdef CFG_DEBUG_USB
                usb_main();
                #endif
                break;
            case 't':
                // timer dump
                tcnt1=TCNT1;
                icr1=ICR1;
                ocr1a=OCR1A;
                ocr1b=OCR1B;
                ocr1c=OCR1C;
                tccr0a=TCCR0A;
                tccr1a=TCCR1A;
                tccr1b=TCCR1B;
                timsk1=TIMSK1;
                printf_P(PSTR("TCNT1=%04X ICR1=%04X OCR1A=%02X OCR1B=%02X OCR1C=%02X TCCR0A=%02X TCCR1A=%02X TCCR1B=%02X TIMSK1=%02X\r\n"),tcnt1,icr1,ocr1a,ocr1b,ocr1c,tccr0a,tccr1a,tccr1b,timsk1);
                #ifdef CFG_DEBUG_USB
                usb_main();
                #endif
                break;
/*
             case 'B':
                // raw buffer dump
                printf_P(PSTR("Rawbuf dump:\r\n"));
                for(uint16_t i=0;i<32;i++)
                    printf_P(PSTR("%02X "),g_sp_buffer[i]);
                printf_P(PSTR("\r\n"));
                #ifdef CFG_DEBUG_USB
                usb_main();
                #endif
                break;
*/
#ifdef CFG_DEBUG_PUSH
            case 'D':
                // debug buffer dump
                printf_P(PSTR("Debug dump:\r\n"));
                for(uint16_t i=0;i<=g_sp_debug_pos;i++)
                {
                    printf_P(PSTR("%02X "),g_sp_debug[i]);
                }
                printf_P(PSTR("\r\n"));                
                memset(g_sp_debug,0,SP_DEBUG_BUF_SIZE);
                g_sp_debug_pos=0;
                #ifdef CFG_DEBUG_USB
                usb_main();
                #endif
                break;
            case 'T':
                // debug buffer dump
                printf_P(PSTR("Debug dump:\r\n"));
                for(uint16_t i=0;i<=g_sp_debug_pos;i++)
                {
                    if(g_sp_debug[i]>=' ' && g_sp_debug[i]<128)
                    {
                        printf_P(PSTR("%c"),g_sp_debug[i]);
                        // don't lost a byte at 19200 with very old hardware
                        _delay_us(500);
                    }
                    else
                    {
                        printf_P(PSTR("<%02X>"),g_sp_debug[i]);
                        // don't lost a byte at 19200 with very old hardware
                        _delay_us(2000);
                    }
                }
                printf_P(PSTR("\r\n"));              
            case 'Y':
                memset(g_sp_debug,0,SP_DEBUG_BUF_SIZE);
                g_sp_debug_pos=0;
                #ifdef CFG_DEBUG_USB
                usb_main();
                #endif
                break;
#endif
            case 'P':
                // packet dump
                printf_P(PSTR("IDdst=%02X IDsrc=%02X PKTtype=%02X PKTauxt=%02X Status=%02X NBodd=%d NBgrp=%d\r\n"),g_sp_packet.header.iddst
                ,g_sp_packet.header.idsrc
                ,g_sp_packet.header.pkttype
                ,g_sp_packet.header.pktauxtype
                ,g_sp_packet.header.status
                ,g_sp_packet.header.nbodd
                ,g_sp_packet.header.nbgrp);
                #ifdef CFG_DEBUG_USB
                usb_main();
                #endif
                for(uint8_t i=0;i<sizeof(g_sp_packet.sync)+1;i++)
                    printf_P(PSTR("%02X "),(((uint8_t*)(&g_sp_packet.sync)))[i]);
                printf_P(PSTR("\r\n"));
                #ifdef CFG_DEBUG_USB
                usb_main();
                #endif
                p=0;
                for(uint8_t i=0;i<sizeof(g_sp_packet.header);i++)
                    printf_P(PSTR("%02X "),(((uint8_t*)(&g_sp_packet.header)))[i]);
                printf_P(PSTR("\r\n"));
                #ifdef CFG_DEBUG_USB
                usb_main();
                #endif
                p=0;
                for(uint8_t i=0;i<(g_sp_packet.header.nbodd&127)+1;i++)
                    printf_P(PSTR("%02X "),(((uint8_t*)(&g_sp_packet.data)))[p++]);
                printf_P(PSTR("\r\n"));
                #ifdef CFG_DEBUG_USB
                usb_main();
                #endif
                for(uint8_t j=0;j<(g_sp_packet.header.nbgrp&127);j++)
                {
                    for(uint8_t i=0;i<8;i++)
                        printf_P(PSTR("%02X "),(((uint8_t*)(&g_sp_packet.data)))[p++]);
                    if((j&1)==1)
                        puts_P(PSTR("\r\n"));
                    else
                        puts_P(PSTR("  "));
                    #ifdef CFG_DEBUG_USB
                    usb_main();
                    #endif
                }
                for(uint8_t i=0;i<4;i++)
                    printf_P(PSTR("%02X "),(((uint8_t*)(&g_sp_packet.data)))[p++]);
                printf_P(PSTR("\r\n"));
                #ifdef CFG_DEBUG_USB
                usb_main();
                #endif
                break;
            case 'p':
                sp_debug_packet();
                break;
/*                
            case 'R':
            	rtc_read(l_tmp);
               printf_P(PSTR("Prodos format %02X %02X %02X %02X\r\n"),l_tmp[0],l_tmp[1],l_tmp[2],l_tmp[3]);
            	break;
            case 'r':
            	rtc_write(l_tmp);
            	break;
*/
				case 'I':
            	ds1307_read(l_tmp);
               printf_P(PSTR("DS1307 read %02X %02X %02X %02X %02X %02X %02X %02X\r\n"),l_tmp[0],l_tmp[1],l_tmp[2],l_tmp[3],l_tmp[4],l_tmp[5],l_tmp[6],l_tmp[7],l_tmp[8]);
            	break;
				case 'i':
					l_tmp[0] = 0;
					l_tmp[1] = 0x12;
					l_tmp[2] = 0x23;
					l_tmp[3] = 0x01;
					l_tmp[4] = 0x10;
					l_tmp[5] = 0x11;
					l_tmp[6] = 0x12;
					l_tmp[7] = 0;
					l_tmp[8] = 0;
					l_tmp[9] = 0;
            	ds1307_write(l_tmp);
               printf_P(PSTR("DS1307 write %02X %02X %02X %02X %02X %02X %02X %02X\r\n"),l_tmp[0],l_tmp[1],l_tmp[2],l_tmp[3],l_tmp[4],l_tmp[5],l_tmp[6],l_tmp[7],l_tmp[8]);
            	break;

        };
    }
#endif
}

void sp_debug_packet(void)
{
#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)

    // packet infos
    //printf_P(PSTR("g_sp_packet.header=%04X\r\ng_sp_state.p_packet=%04X"),&g_sp_packet,&g_sp_packet.header,g_sp_state.p_packet);
    puts_P(PSTR("\r\nactivity="));
    switch(g_sp_state.activity)
    {
        case SP_RECEIVE:
            puts_P(PSTR("receive"));
            break;          
        case SP_SEND:
            puts_P(PSTR("send"));
            break;
    }
    printf_P(PSTR("\r\n step_rec="));
    switch(g_sp_state.receive_step)
    {
        case SP_STOP:
            puts_P(PSTR("STOP"));
            break;
        case SP_WAIT:
            puts_P(PSTR("WAIT"));
            break;
        case SP_SYNC:
            puts_P(PSTR("SYNC"));
            break;
        case SP_HEAD:
            puts_P(PSTR("HEAD"));
            break;
        case SP_DATA_ODD_MSB:
            puts_P(PSTR("DATA ODD MSB"));
            break;
        case SP_DATA_ODD:
            puts_P(PSTR("DATA ODD"));
            break;
        case SP_DATA_GRP_MSB:
            puts_P(PSTR("DATA GRP MSB"));
            break;
        case SP_DATA_GRP:
            puts_P(PSTR("DATA GRP"));
            break;
        case SP_CRC:
            puts_P(PSTR("CRC"));
            break;
        case SP_END:
            puts_P(PSTR("END"));
            break;
    }
    #ifdef CFG_DEBUG_USB
    usb_main();
    #endif
    puts_P(PSTR("\r\n step_snd="));
    switch(g_sp_state.send_step)
    {
        case SP_STOP:
            puts_P(PSTR("STOP"));
            break;
        case SP_WAIT:
            puts_P(PSTR("WAIT"));
            break;
        case SP_SYNC:
            puts_P(PSTR("SYNC"));
            break;
        case SP_HEAD:
            puts_P(PSTR("HEAD"));
            break;
        case SP_DATA_ODD_MSB:
            puts_P(PSTR("DATA ODD MSB"));
            break;
        case SP_DATA_ODD:
            puts_P(PSTR("DATA ODD"));
            break;
        case SP_DATA_GRP_MSB:
            puts_P(PSTR("DATA GRP MSB"));
            break;
        case SP_DATA_GRP:
            puts_P(PSTR("DATA GRP"));
            break;
        case SP_CRC:
            puts_P(PSTR("CRC"));
            break;
        case SP_END:
            puts_P(PSTR("END"));
            break;
    }
    #ifdef CFG_DEBUG_USB
    usb_main();
    #endif
    puts_P(PSTR("\r\nerror="));
    switch(g_sp_state.error)
    {
        case SP_ERROK:
            puts_P(PSTR("NO ERR"));
            break;
        case SP_ERRCRC:
            puts_P(PSTR("CRC ERR"));
            break;
        case SP_ERRTIMEOUT:
            puts_P(PSTR("TIMEOUT"));
            break;
        case SP_ERROVERFLOW:
            puts_P(PSTR("OVERFLOW ERR"));
            break;
        case SP_ERRPACKET:
            puts_P(PSTR("PACKET ERR"));
            break;
        case SP_ERRRESET:
            puts_P(PSTR("RESET"));
            break;
        case SP_ERRTIMING:
            puts_P(PSTR("TIMING ERR"));
            break;
    }
    #ifdef CFG_DEBUG_USB
    usb_main();
    #endif
    //printf_P(PSTR("\r\ncrc comp=%02X\r\ncrc rec=%02X"),g_sp_state.crc_computed,g_sp_state.crc_received);
    //printf_P(PSTR("\r\nsiz comp=%d\r\nsiz rec=%d"),g_sp_state.size_computed,g_sp_state.size_received);
    printf_P(PSTR("\r\ngp_sp_sisr=%04X gp_sp_sisr_end=%04X\r\n"),gp_sp_sisr,gp_sp_sisr_end);
    #ifdef CFG_DEBUG_USB
    usb_main();
    #endif
#endif
}
