;=============================================================================
;Smartport Virtual Hard Drive, version 1.00.00
;isr to send data
;
;If you would like to be involved in this project, or simply get my 
;documentation for your own smartport project, feel free to contact me at
;pcedric@hotmail.com
;
;Copyright 2010,2011,2012 Cédric PELTIER, pcedric@hotmail.com
;
;This file is part of the Smartport VHD project
;
;Smartport VHD is free software: you can redistribute it and/or modify
;it under the terms of the GNU General Public License as published by
;the Free Software Foundation version 3
;
;Smartport VHD is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with Smartport VHD. If not, see <http://www.gnu.org/licenses/>.
;=============================================================================


#include <avr/interrupt.h>
#include <avr/io.h>
#include "protocol.h"

#if !defined(CFG_SP_ISR_USE_STACK) && !defined(CFG_SP_ISR_USE_GPIOR) && !defined(CFG_SP_ISR_USE_TIMER0) && !defined(CFG_SP_ISR_USE_IOR)  && !defined(CFG_SP_ISR_USE_DEDICATED)  
    #error No method defined to save register in ISR !
#endif

;asm receive isr is no more used and kept only for reference, check the C isr handle instead
#if false
;Raised at each bit set to one from the apple 2 on the WR line
.global TIMER1_CAPT_vect
TIMER1_CAPT_vect:                           ;4/5
#ifdef CFG_SP_ISR_USE_STACK
    push r26                                ;2
    push r27                                ;2
    push r0                                 ;2
    push r3                                 ;2
    in   r0,_SFR_IO_ADDR(SREG)              ;1
;9
#endif
#ifdef CFG_SP_ISR_USE_GPIOR
    push r26                                ;2
    out  _SFR_IO_ADDR(GPIOR2),r27           ;1
    out  _SFR_IO_ADDR(GPIOR0),r0            ;1
    out  _SFR_IO_ADDR(GPIOR1),r3            ;1
    in   r0,_SFR_IO_ADDR(SREG)              ;1
;6
#endif
#ifdef CFG_SP_ISR_USE_TIMER0
    push r26                                ;2
    out  _SFR_IO_ADDR(TCNT0),r27            ;1
    out  _SFR_IO_ADDR(OCR0A),r0             ;1
    out  _SFR_IO_ADDR(OCR0B),r3             ;1
    in   r0,_SFR_IO_ADDR(SREG)              ;1
;6
#endif
#ifdef CFG_SP_ISR_USE_IOR
    out  _SFR_IO_ADDR(OCR0B),r26            ;1
    out  _SFR_IO_ADDR(GPIOR2),r27           ;1
    out  _SFR_IO_ADDR(GPIOR0),r0            ;1
    out  _SFR_IO_ADDR(GPIOR1),r3            ;1
    in   r0,_SFR_IO_ADDR(SREG)              ;1
;5
#endif
#ifdef CFG_SP_ISR_USE_DEDICATED
    in   r2,_SFR_IO_ADDR(SREG)              ;1  
#endif

;receive data only if smartport is enabled and req line is high
;don't let the upper layer manage it !
;    in   r26,_SFR_IO_ADDR(PINF)             ;1
;    andi r26,(SP_IREQ|SP_ICA1|SP_ICA2|SP_ILSTRB) ;1
;    cpi  r26,(SP_IREQ|SP_ICA1        |SP_ILSTRB) ;1
;    brne t1c_end                            ;1/2
;4/5

;update the edge detection
;ICES1=0 to detect a falling edge
;ICES1=1 to detect a raising edge
;isr_state = current value of TCCR1B
;isr_bit = (1<<ICES1)
;    eor  g_sp_isr_state,g_sp_isr_bit       ;1
;    sts  TCCR1B,g_sp_isr_state             ;2
;3

;slower but freeing two registers...
    lds  r26,TCCR1B                         ;2
    ldi  r27,(1<<ICES1)                     ;1
    eor  r26,r27                            ;1
    sts  TCCR1B,r26                         ;2
;6

;write the new timestamp into the ring buffer
; move pointer into the X index to write
; into the ring buffer
#ifdef CFG_RB_8BIT
    lds  r26,g_sp_risr                      ;2
    ldi  r27,hi8(SP_RING_START)             ;1
;3
#else
    lds  r26,gp_sp_risr                     ;2
    lds  r27,gp_sp_risr+1                   ;2
; check the end of the ring buffer
    cpi  r27,hi8(SP_RING_END)               ;1
    brne t1c_store                          ;1/2
    cpi  r26,lo8(SP_RING_END)               ;1
    brne t1c_store                          ;1/2
    ldi  r26,lo8(SP_RING_START)     ;1
    ldi  r27,hi8(SP_RING_START)     ;1
;8/11
#endif

t1c_store:
; then store the timestamp and increment the X index
    lds  r3,ICR1L                           ;2
    st   x+,r3                              ;2/3 (xmem)
;4/5
; using the /8 prescaler, only the low byte is required
;    lds  r3,ICR1H                           ;2
;    st   x+,r3                              ;2/3 (xmem)
; save the new value for the pointer
#ifdef CFG_RB_8BIT
    sts  g_sp_risr,r26                      ;2
;2
#else
    sts  gp_sp_risr,r26                     ;2
    sts  gp_sp_risr+1,r27                   ;2
;4
#endif

t1c_end:

;debug only
;    ldi  r26,(1<<PD7)
;    out  _SFR_IO_ADDR(PIND),r26

#ifdef CFG_SP_ISR_USE_STACK
    out  _SFR_IO_ADDR(SREG),r0              ;1
    pop  r3                                 ;2
    pop  r0                                 ;2
    pop  r27                                ;2
    pop  r26                                ;2
;9
#endif
#ifdef CFG_SP_ISR_USE_GPIOR
    out  _SFR_IO_ADDR(SREG),r0              ;1
    in   r3,_SFR_IO_ADDR(GPIOR1)            ;1
    in   r0,_SFR_IO_ADDR(GPIOR0)            ;1
    in   r27,_SFR_IO_ADDR(GPIOR2)           ;1
    pop  r26                                ;2
;6
#endif
#ifdef CFG_SP_ISR_USE_TIMER0
    out  _SFR_IO_ADDR(SREG),r0              ;1
    in   r3,_SFR_IO_ADDR(OCR0B)             ;1
    in   r0,_SFR_IO_ADDR(OCR0A)             ;1
    in   r27,_SFR_IO_ADDR(TCNT0)            ;1
    pop  r26                                ;2
;6
#endif
#ifdef CFG_SP_ISR_USE_IOR
    out  _SFR_IO_ADDR(SREG),r0              ;1
    in   r3,_SFR_IO_ADDR(GPIOR1)            ;1
    in   r0,_SFR_IO_ADDR(GPIOR0)            ;1
    in   r27,_SFR_IO_ADDR(GPIOR2)           ;1
    in   r26,_SFR_IO_ADDR(OCR0B)            ;1
;5
#endif
#ifdef CFG_SP_ISR_USE_DEDICATED
    out  _SFR_IO_ADDR(SREG),r2              ;1  
#endif

    reti                                    ;4

                                            ;4+1+0+6+3+4+2+1+4=25
#endif


;setup the timer1 to generate a pulse of 1us width at 4us interval
;setup is done in sp_send_init()
;the interrupt at each end of pulse allow to connect or disconnect
;the output pin from the timer control for the next pulse.
;g_sp_sisr_bit is the current bit to be sent and must be set to 0x80
;g_sp_sisr_byte must be set with the first byte to send
;gp_sp_sisr must be set to the start of the buffer to send +1
;gp_sp_sisr_end must be set to the end of the buffer to send +1
.global TIMER1_COMPC_vect
TIMER1_COMPC_vect:								  ;5
#ifdef CFG_SP_ISR_USE_STACK
    push r26                                ;2
    push r27                                ;2
    push r0                                 ;2
    push r16                                ;2
    in   r0,_SFR_IO_ADDR(SREG)              ;1
;9
#endif
#ifdef CFG_SP_ISR_USE_GPIOR
    push r26                                ;2
    out  _SFR_IO_ADDR(GPIOR2),r27           ;1
    out  _SFR_IO_ADDR(GPIOR0),r0            ;1
    out  _SFR_IO_ADDR(GPIOR1),r16           ;1
    in   r0,_SFR_IO_ADDR(SREG)              ;1
;6
#endif
#ifdef CFG_SP_ISR_USE_TIMER0
    push r26                                ;2
    out  _SFR_IO_ADDR(TCNT0),r27            ;1
    out  _SFR_IO_ADDR(OCR0A),r0             ;1
    out  _SFR_IO_ADDR(OCR0B),r16            ;1
    in   r0,_SFR_IO_ADDR(SREG)              ;1
;6
#endif
#ifdef CFG_SP_ISR_USE_IOR
    out  _SFR_IO_ADDR(OCR0B),r26            ;1
    out  _SFR_IO_ADDR(GPIOR2),r27           ;1
    out  _SFR_IO_ADDR(GPIOR0),r0            ;1
    out  _SFR_IO_ADDR(GPIOR1),r16           ;1
    in   r0,_SFR_IO_ADDR(SREG)              ;1
;5
#endif

;debug only
;    ldi  r16,(1<<PD7)
;    out  _SFR_IO_ADDR(PIND),r16

;check the current bit
    ldi  r27,(1<<WGM11)                     ;1
;load the current byte to send
    lds  r26,g_sp_sisr_byte                 ;2
;load the shifting counter
    lds  r16,g_sp_sisr_bit                  ;2
    and  r26,r16                            ;1
    breq t1c_snd_updatepulse                ;1/2
;activate the pin output to enable the next pulse
    ori  r27,(1<<COM1C1)|(1<<COM1C0)        ;1

t1c_snd_updatepulse:
;activate or deactivate the next pulse
    sts  TCCR1A,r27                         ;2
; update shifting counter
    lsr  r16                                ;1
    brne t1c_snd_end                        ;1/2
;12/13

;at each byte sent,
;check the state of the smartport to ensure it's
;listening, else mute the rd line
;no, let's the upper layer manage it !
;    in    r26,_SFR_IO_ADDR(PINF)            ;1
;    andi  r26,(SP_ICA1|SP_ICA2|SP_ILSTRB)   ;1 SP_IREQ|
;    cp    r26,(SP_ICA1|SP_ILSTRB)           ;1 SP_IREQ|
;    breq  t1c_snd_ok                        ;1/2
;mute the rd line now !!!
;    cbi  _SFR_IO_ADDR(DDRB),SP_ORDD_BN      ;2

;debug only
;    ldi  r16,(1<<PB3)
;    out  _SFR_IO_ADDR(PINB),r16

;t1c_snd_ok:
;+5

; load the reading pointer into x register
    lds  r26,gp_sp_sisr                     ;2
    lds  r27,gp_sp_sisr+1                   ;2
;4

; check the end of the buffer
    lds  r16,gp_sp_sisr_end                 ;2
    cp   r26,r16                            ;1
    lds  r16,gp_sp_sisr_end+1               ;2
    cpc  r27,r16                            ;1
    brsh t1c_snd_stop                       ;1/2
;7/8

;read the next byte
; read the byte and increment the pointer
    ld   r16,x+                             ;2/3 (xmem)
; save the byte to send the remaining bits
; without using pointers for each of them
    sts  g_sp_sisr_byte,r16                 ;2
; save the new value for the pointer
    sts  gp_sp_sisr,r26                     ;2
    sts  gp_sp_sisr+1,r27                   ;2
;reset the shifting counter
    ldi  r16,0x80                           ;1
;10

t1c_snd_end:
;save the shifting counter
    sts  g_sp_sisr_bit,r16                  ;2

#ifdef CFG_SP_ISR_USE_STACK
    out  _SFR_IO_ADDR(SREG),r0              ;1
    pop  r16                                ;2
    pop  r0                                 ;2
    pop  r27                                ;2
    pop  r26                                ;2
;9
#endif
#ifdef CFG_SP_ISR_USE_GPIOR
    out  _SFR_IO_ADDR(SREG),r0              ;1
    in   r16,_SFR_IO_ADDR(GPIOR1)           ;1
    in   r0,_SFR_IO_ADDR(GPIOR0)            ;1
    in   r27,_SFR_IO_ADDR(GPIOR2)           ;1
    pop  r26                                ;2
;6
#endif
#ifdef CFG_SP_ISR_USE_TIMER0
    out  _SFR_IO_ADDR(SREG),r0              ;1
    in   r16,_SFR_IO_ADDR(OCR0B)            ;1
    in   r0,_SFR_IO_ADDR(OCR0A)             ;1
    in   r27,_SFR_IO_ADDR(TCNT0)            ;1
    pop  r26                                ;2
;6
#endif
#ifdef CFG_SP_ISR_USE_IOR
    out  _SFR_IO_ADDR(SREG),r0              ;1
    in   r16,_SFR_IO_ADDR(GPIOR1)           ;1
    in   r0,_SFR_IO_ADDR(GPIOR0)            ;1
    in   r27,_SFR_IO_ADDR(GPIOR2)           ;1
    in   r26,_SFR_IO_ADDR(OCR0B)            ;1
;5
#endif
    reti                                    ;5

;41 x7  67%
;64 x1 100%
;351/512 68%

;35 x7  55%
;58 x1  91%
;351/512 59%

                                            ;31 cycles for 7 bits (47%)
                                            ;41 +5 cycles for 1 bit (62.5%)
                                            ;31.25 cycles average for each byte (49%)

                                            ;20 cycles for 7 bits (31.25% of cpu power @16MHz)
                                            ;28 +5 cycles for 1 bit (43.75% of cpu power @16MHz)
                                            ;21 cycles average for each byte (32.81%)

;no more byte to send
t1c_snd_stop:

;both start the handshake with the host
;and signal the end to the upper layer
    cbi  _SFR_IO_ADDR(SP_OACK_PORT),SP_OACK_BN     ;2
    sbi  _SFR_IO_ADDR(SP_OACK_DDR),SP_OACK_BN		;2
    
    ldi  r16,0                              ;1
;disconnect the rd line from the counter control
;   sts  TCCR1A,r16                         ;2
;mute the rd line, no more data to send
;   cbi  _SFR_IO_ADDR(DDRB),SP_ORDD_BN      ;2
;stop the timer clock
    sts  TCCR1B,r16                         ;2
;stop the interrupt
    sts  TIMSK1,r16                         ;2
    rjmp t1c_snd_end                        ;2