/*
=============================================================================
Smartport Virtual Hard Drive, version 0.99.0 (beta)
main source file

If you would like to be involved in this project, or simply get my 
documentation for your own smartport project, feel free to contact me at
pcedric@hotmail.com

Copyright 2010 Cédric PELTIER, pcedric@hotmail.com

This file is part of the Smartport VHD project

Smartport VHD is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 3

Smartport VHD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Smartport VHD. If not, see <http://www.gnu.org/licenses/>.
=============================================================================
*/




#include "spvhd.h"
#include "protocol.h"
#include "cache.h"
#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>
#include <LUFA/Drivers/USB/USB.h>
#include "extsram.h"
#ifdef CFG_DEBUG_SERIAL
   #if(LUFA_VERSION==100219)
	#include <LUFA/Drivers/Peripheral/SerialStream.h>
	#else
	#include <LUFA/Drivers/Peripheral/Serial.h>
	#endif
#endif
#ifdef CFG_DEBUG_USB
	// using the usb port to debug
	#include "usb_serial.h"
	#ifndef USB_CAN_BE_DEVICE
		#error USB_CAN_BE_DEVICE is not declared, something goes wrong...
	#endif
	extern USB_ClassInfo_CDC_Device_t VirtualSerial_CDC_Interface;
#else
	// using the usb port for mass storage
	#include "usb_storage.h"
	#ifndef USB_CAN_BE_HOST
		#error USB_CAN_BE_HOST is not declared, something goes wrong...
	#endif
	extern USB_ClassInfo_MS_Host_t FlashDisk_MS_Interface;
#endif
#include "Lib/FATFs/ff.h"




/// led timer for simple blinking
uint8_t g_tc_led_low;
uint16_t g_tc_led_high;




/// Main entry point
int main(void)
{
/*
	uint8_t l_rc;
	uint8_t i;
    DDRA = 0;
    PORTA = 0;
    DDRB = 0; 
    PORTB = 0;
    DDRC = 0; 
    PORTC = 0;
    DDRD = 0;
    PORTD = 0;
    DDRE = 0;
    PORTE = 0;
    DDRF = 0;
    PORTF = 0;
    xr_init();
    CSET_BANK_SET
    uint8_t l_pd;
    uint8_t l_pf;
	 uint8_t* lp_ptr = EXT_SRAM_START;
	 l_pd = PINB;
	 l_pf = PIND;
	 *lp_ptr = l_pd;
	 lp_ptr++;	 
	 *lp_ptr = l_pf;
	 lp_ptr++;	 
	 while(l_pd==PIND && l_pf==PINF);
	 while(lp_ptr<EXT_SRAM_END-2)
	 {
		 l_pd = PINB;
		 l_pf = PIND;
		 *lp_ptr = l_pd;
		 lp_ptr++;	 
		 *lp_ptr = l_pf;
		 lp_ptr++;
		 _delay_us(3);
	 }

		usb_init(0);
		// Wait at least 3 seconds to detect the usb stick
		uint16_t l_count = 3000;
		// Wait at least 3 seconds more if something is attached to the usb
		uint16_t l_delay = 7000;
		// Blinking counter
		uint16_t l_blink = 0;
		while(USB_HostState != HOST_STATE_Configured)
		{
			// if an usb device is present, allow more time for enumeration
			if(USB_HostState!=HOST_STATE_Unattached && l_delay !=0)
			{
				l_count += l_delay;
				l_delay = 0;
			}
			// blink the power led on at least 1/4 second basis max until the usb enumeration start
			if(l_blink++>500)
			{
				l_blink=0;
			}
			if(l_blink>250)
			{
				LEDs_TurnOffLEDs(LEDMASK_POWER);
			}
			else
			{
				LEDs_TurnOnLEDs(LEDMASK_POWER);
			}
			// continue usb initialiation until the usb enumeration start
			usb_init(1);
			if(l_count--==0 && USB_HostState != HOST_STATE_Configured)
			{
				// timeout, no storage available...
				LEDs_TurnOffLEDs(LEDMASK_POWER);
				die(LEDMASK_ERROR,LEDMASK_ERROR,LEDMASK_ERROR,1000);			
			}
			_delay_ms(1);
		}
		LEDs_TurnOffLEDs(LEDMASK_POWER);
	
		// usb comunication ready with the device, try to mount the file system
		FRESULT rc = f_mount(0,&g_sp_state.filesystem);
		if(rc!=FR_OK)
		{
			#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
			if(gb_debug & CFG_DEBUG_MASK_PROTO)	printf_P(PSTR("FAT: unable to mount the file system\r\n"));
			#endif
			sp_init_drives_error(PSTR("No file system"));
			die(LEDMASK_ERROR,LEDMASK_ERROR,0,1000);			
		}
		uint8_t lp_filename[32];
		FIL* lp_fh;
		uint8_t* lp_dst;
		uint8_t k;
		strcpy_P(lp_filename,PSTR("0:/snif.bin"));
		l_rc = f_open(lp_fh,lp_filename,FA_CREATE_NEW | FA_WRITE);
		if(l_rc != FR_OK)
			die(LEDMASK_ERROR,0,0,1000);
		lp_ptr = EXT_SRAM_START;
		while(lp_ptr<EXT_SRAM_END)
		{
			lp_dst = &g_sp_packet.data[0];
			for(k=0;k<31 && lp_ptr<EXT_SRAM_END-2;k++)
			{
				uint8_t j = *lp_ptr;
				for(i=0;i<8;i++)
				{
					*lp_dst = ((j&128)==0?'0':'1');
					j = j<1;
					lp_dst++;
				}
				*lp_dst = ' ';
				lp_dst++;
				lp_ptr++;
				j = *lp_ptr;
				for(i=0;i<8;i++)
				{
					*lp_dst = ((j&128)==0?'0':'1');
					j = j<1;
					lp_dst++;
				}
				*lp_dst = "\r";
				lp_dst++;
				*lp_dst = "\n";
				lp_dst++;
				lp_ptr++;
			}
				
			UINT l_size_write;
			l_rc = f_write(lp_fh,&g_sp_packet.data[0],31*(16+3),&l_size_write);
			if(l_rc != FR_OK)
				die(LEDMASK_ERROR|LEDMASK_POWER,0,0,1000);
		}
		f_close(lp_fh);

		die(LEDMASK_POWER,LEDMASK_POWER,LEDMASK_POWER,500);
*/		
	 
    init();
    for (;;)
    {
		usb_main();
		sp_main();
	}

}

/// All initialiszations are called from here
void init()
{
    g_tc_led_low=0;
    g_tc_led_high=0;

    /* Disable watchdog if enabled by bootloader/fuses */
    MCUSR &= ~(1 << WDRF);
    wdt_disable();

    /* Disable clock division */
    clock_prescale_set(clock_div_1);

    /* disable JTAG to allow corresponding pins to be used - PF4, PF5, PF6, PF7 */
    /* TODO - remove this if you want to use your JTAG debugger to debug this firmware */
    // note the JTD bit must be written twice within 4 clock cycles to disable JTAG
    // you must also set the IVSEL bit at the same time, which requires IVCE to be set first
    // port pull-up resistors are enabled - PUD(Pull Up Disable) = 0
    MCUCR = (1 << JTD) | (1 << IVCE) | (0 << PUD);
    MCUCR = (1 << JTD) | (0 << IVSEL) | (0 << IVCE) | (0 << PUD);

    // set all ports 
    // ODR2 is high output by default to disable daisy chain disk II -> NO!
    // OINT is high output by default to disable second daisy chain Unidisk 5.25
    // IHDSEL is input with pullup
    DDRA = 0;
    PORTA = 0;
    DDRB = SP_OPORTB; 
    PORTB = 0;//SP_ODR2;
    DDRC = 0; 
    PORTC = 0;
    DDRD = SP_OPORTD;
    PORTD = SP_IHDSEL;
    DDRE = SP_OPORTE|(1<<PE3)|(1<<PE4);
    PORTE = 0;//SP_IINT;
    DDRF = SP_OPORTF;
    PORTF = 0;
  
    #ifdef CFG_DEBUG_SERIAL
    // initialize the serial port for debug purpose
    #if(LUFA_VERSION==100219)
	    SerialStream_Init(115200,false);
	 #else
	    Serial_Init(115200,false);
	 	 Serial_CreateStream(NULL);
	 #endif
	 puts_P(PSTR("\r\nStarted\r\n"));
    #endif
    
    // initialize leds
    LEDs_Init();
    LEDs_SetAllLEDs(0);
    
    // initialize external ram and the shadowing system
    xr_init();
    CSET_BANK_SET
    cset_init();
    CSET_BANK_UNSET

    // initialize the protocol converter layer 
    sp_init_cold();
 	
	// Adding rtc device for test purpose: not recognized by ProDOS !
	/*
	i = g_sp_state.drive_max;
	g_sp_state.drive_max++;
	memcpy_P(&g_sp_state.dib[i][0],g_sp_dib_template,SP_DIBSIZE);
	g_sp_state.dib[i][0] = 0x70;
	g_sp_state.dib[i][1] = 0;
	g_sp_state.dib[i][2] = 0;
	g_sp_state.dib[i][3] = 0;
	sprintf(&g_sp_state.dib[i][5],"Clock",i);
	g_sp_state.dib[i][4]=strlen(&g_sp_state.dib[i][5]);
	g_sp_state.dib[i][g_sp_state.dib[i][4]+5]=' ';
	g_sp_state.dib[i][21] = 0x0E;
	*/
	
	// Test only
    /*
    for(UINT k=0;k<100*512;k+=512)
	{
		FRESULT rc = f_lseek(&g_sp_state.drive[0],k);
		if(rc==FR_OK)
		{
			UINT size_read=0;
			rc = f_read(&g_sp_state.drive[0],g_sp_buffer,512,&size_read);
			if(size_read != 512)
				die(LEDMASK_POWER,LEDMASK_WARNING,LEDMASK_ERROR,100);				
		}
		if(rc!=FR_OK)
			die(LEDMASK_POWER,LEDMASK_WARNING,LEDMASK_ERROR,250);
		uint16_t p = 0;
		for(uint8_t i=0;i<32;i++)
		{
			printf_P(PSTR("%04X:"),k+i*16);
			for(uint8_t j=0;j<16;j++)
				printf_P(PSTR("%02X "),(((uint8_t*)(&g_sp_buffer)))[p++]);
			puts_P(PSTR("\r"));
		}
		#ifdef CFG_DEBUG_USB
		usb_main();
		#endif
	}
    */
    #ifdef CFG_DEBUG_PUSH
    memset((uint8_t*)SP_DEBUG_BUF,0,SP_DEBUG_BUF_SIZE);
    #endif
    
    // start listening the smartport bus
    sp_receive_init();
}

/// Blink leds
/// @param uint8_t leds Led mask to use
/// @param uint8_t times Number of times to togle leds
/// @param uint1-_t delay Delay between each toggle in ms
void blink(uint8_t l_leds,uint8_t l_times,uint16_t l_delay)
{
	uint8_t i;
	for(i=0;i<l_times;i++)
	{
		LEDs_ToggleLEDs(l_leds);
		_delay_ms(l_delay);
	}
}

/// Die with blinking leds series
void die(uint8_t l_leds1,uint8_t l_leds2,uint8_t l_leds3,uint16_t l_delay)
{
    for(;;)
    {
    	LEDs_SetAllLEDs(0);
    	_delay_ms(1000); 
        LEDs_SetAllLEDs(l_leds1);
        _delay_ms(l_delay);
    	LEDs_SetAllLEDs(0);
    	_delay_ms(100); 
        LEDs_SetAllLEDs(l_leds2);
        _delay_ms(l_delay);
    	LEDs_SetAllLEDs(0);
    	_delay_ms(100); 
        LEDs_SetAllLEDs(l_leds3);
        _delay_ms(l_delay);
    	LEDs_SetAllLEDs(0);
        
    	#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
        // the following must be called to allow debuging after dying
        #if defined(CFG_DEBUG_USB)
    	usb_main();
        #endif
        sp_debug();
        #endif
    }
}

/// Read or write a block of data from or to a configuration file
/// @param lp_filename Filename of the configuration file
/// @param l_number Block number to be read or write
/// @param l_rw =0 to read from the file, =1 to write to the file
/// @param lp_buffer Pointer to the buffer holding the data
/// @return 0 if success, 1 if the operation failed 
uint8_t cfg_file_io(uint8_t* lp_filename, uint8_t l_number,uint8_t l_rw,uint8_t* lp_buffer)
{
	FIL l_fh;
	FRESULT l_rc;
	UINT l_position = (UINT)(l_number) * (UINT)(SPVHD_CFG_BLOCKSIZE);
	#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
	if(gb_debug & CFG_DEBUG_MASK_SPVHD)	  printf_P(PSTR("SPB: cfg_file_io(%s,%d) pos=%04X\r\n"),lp_filename,l_number,l_position);
	#endif
	l_rc = f_open(&l_fh, (const TCHAR*)lp_filename,FA_OPEN_EXISTING | (l_rw==0?FA_READ:FA_WRITE));
	if(l_rc == FR_OK)
	{
		#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
		if(gb_debug & CFG_DEBUG_MASK_SPVHD)		  puts_P(PSTR("SPB: opened\r\n"));
		#endif
		l_rc = f_lseek(&l_fh,l_position);
		if(l_rc==FR_OK)
		{
			UINT l_size;
		   CSET_BANK_SET;
			if(l_rw==0)
				l_rc = f_read(&l_fh,(const void*)lp_buffer,(UINT)SPVHD_CFG_BLOCKSIZE,&l_size);
			else
				l_rc = f_write(&l_fh,(const void*)lp_buffer,(UINT)SPVHD_CFG_BLOCKSIZE,&l_size);
			CSET_BANK_UNSET;
			#if defined(CFG_DEBUG_USB) || defined(CFG_DEBUG_SERIAL)
			if(gb_debug & CFG_DEBUG_MASK_SPVHD)				printf_P(PSTR("SPB: read/write return %d\r\n"),l_rc);
		   #endif
		}
		f_close(&l_fh);
	}
	return (l_rc==FR_OK?0:1);
}


