/*
=============================================================================
Smartport Virtual Hard Drive, version 0.99.0 (beta)
main source file

If you would like to be involved in this project, or simply get my 
documentation for your own smartport project, feel free to contact me at
pcedric@hotmail.com

Copyright 2010 Cédric PELTIER, pcedric@hotmail.com

This file is part of the Smartport VHD project

Smartport VHD is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 3

Smartport VHD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Smartport VHD. If not, see <http://www.gnu.org/licenses/>.
=============================================================================
*/

#ifndef _SPVHD_H_
#define _SPVHD_H_

	/// To comply with the latest avr libc, but deprecated 
	#define __DELAY_BACKWARD_COMPATIBLE__
	
// =============================================================================
// Dependencies

    // general includes
    #include <avr/io.h>
    #include <avr/wdt.h>
    #include <avr/interrupt.h>
    #include <avr/power.h>
    #include <stdbool.h>
    #include <string.h>
    #include <stdio.h>

    // LUFA related
#ifdef CFG_DEBUG_USB
    #include "usb_serial_desc.h"
#else
//    #include "usb_storage_desc.h"
#endif
    #include <LUFA/Version.h>
    #include <LUFA/Drivers/Board/LEDs.h>
    //#include <LUFA/Drivers/Board/Joystick.h>
    //#include <LUFA/Drivers/Board/Buttons.h>
    #include "Lib/FATFs/ff.h"

// =============================================================================
// Hardware related

    /// power led (green)
    #define LEDMASK_POWER    LEDS_LED1

    /// warning led (orange)
    #define LEDMASK_WARNING  LEDS_LED2

    /// error led (red)
    #define LEDMASK_ERROR    LEDS_LED3

    /// LED mask for the library LED driver, all leds.
    #define LEDMASK_ALL      (LEDMASK_POWER | LEDMASK_WARNING | LEDMASK_ERROR)
    
    /// LED indicator related to the protocol
    #define LEDMASK_SP		LEDMASK_WARNING
    
    /// LED indicator related to usb
    #define LEDMASK_USB		LEDMASK_ERROR
    
    /// LED indicator related to the hardware
    #define LEDMASK_HDW		LEDMASK_POWER
    
    /// LED timer mask for quick blinking
    #define LEDMASK_TIMER_QUICK	64

    /// LED timer mask for normal blinking
    #define LEDMASK_TIMER_NORMAL	256

    /// LED timer mask for normal blinking
    #define LEDMASK_TIMER_SLOW	1024

// =============================================================================
// Debug flags
	
	#define CFG_DEBUG_MASK_CACHE	1
	#define CFG_DEBUG_MASK_RCV		2
	#define CFG_DEBUG_MASK_SND		4
	#define CFG_DEBUG_MASK_PROTO	8
	#define CFG_DEBUG_MASK_SPVHD	16
	#define CFG_DEBUG_MASK_BUS		32
	#define CFG_DEBUG_MASK_USB		64
	#define CFG_DEBUG_MASK_RTC		128
	#define CFG_DEBUG_MASK_I2C		256
	#define CFG_DEBUG_MASK_RWTS	512
    
// =============================================================================
// SmartportVHD firmware calls

    /// status and control codes dedicated to the smartportvhd
    #define SPVHD_DATE 			10	// date
    #define SPVHD_EXPLORE		11 // set pathname to explore the repository
    #define SPVHD_NEXTFILE		12 // get next file name from the repository
    #define SPVHD_PARMSEL		13 // virtual device selector for parameters access
    #define SPVHD_PARMBOOT	   14 // disk image pathname at cold boot for a vitual device
    #define SPVHD_PARMTEMP	   15 // disk image pathname to use now for a virtual device
    #define SPVHD_BLOCK			255 // data block to write into image disk
   
    #define SPVHD_VERSION_MAJOR	1
    #define SPVHD_VERSION_MINOR	0
    #define SPVHD_RESVISION		0
    
    #define SPVHD_CFG_BLOCKSIZE 128
    
// =============================================================================
// SmartportVHD firmware structures
    
	/// device control block (smartportvhd specific)
	struct spvhd_dcb
	{
	  // major version number
	  uint8_t major;
	  // minor version number
	  uint8_t minor;
	  // reversion number
	  uint8_t revision;
	  // smartport id
	  uint8_t spid;
	  // number of devices
	  uint8_t devicecount;
	  // size of the following string
	  uint8_t signaturesize;
	  // identification string
	  uint8_t signature[5];
	};

	/// structure used with call SPVHD_EXPLORE
	struct spvhd_explore
	{
		// size of the following string 
		uint8_t pathsize;
		// disk image pathname
		uint8_t path[127];
	};

	/// structure used with call SPVHD_NEXTFILE
	struct spvhd_nextfile
	{
		// file size in bytes
		uint32_t filesize;
		// file attribute
		uint8_t fileattr;
		// size of the filename 
		uint8_t namesize;
		// file name followed by formated file size on 10 characters
		uint8_t name[13+10+1];
	};
	
	/// structure used with calls SPVHD_PARMBOOT and SPVHD_PARMTEMP
	struct spvhd_parms
	{
		// parameters byte
		uint8_t parms;
		// size of the following string 
		uint8_t pathsize;
		// disk image pathname
		uint8_t path[125];
	};

	/// structure used with call SPVHD_DATE
	struct spvhd_date
	{
		// encoded date and hours
		uint8_t date[4];
	};
	
// =============================================================================
// Functions Prototypes
    
    void init(void);
    //void update_passtrough(void);
    void leds(void);
    void blink(uint8_t leds,uint8_t times,uint16_t delay);
    void die(uint8_t leds1,uint8_t leds2,uint8_t leds3,uint16_t delay);
    uint8_t cfg_file_io(uint8_t* lp_filename, uint8_t l_number,uint8_t l_rw,uint8_t* lp_buffer);

#endif
